import io
import os
import subprocess

import PIL
from PIL import ImageQt, Image, ImageFilter, ImageGrab
from PIL.ImageQt import QImage
from PySide6.QtCore import Qt, QPoint
from PySide6.QtGui import QEnterEvent, QDragEnterEvent, QDropEvent, QImageReader, QDragMoveEvent, QMouseEvent
from PySide6.QtGui import QPixmap, QPainter
from PySide6.QtWidgets import QGraphicsScene, QGraphicsPixmapItem, QFileDialog, QGraphicsSceneMouseEvent

from airunner.aihandler.logger import Logger
from airunner.enums import SignalCode, CanvasToolName, GeneratorSection, EngineResponseCode
from airunner.mediator_mixin import MediatorMixin
from airunner.settings import VALID_IMAGE_FILES
from airunner.utils import platform_info
from airunner.utils.export_image import export_image
from airunner.utils.snap_to_grid import snap_to_grid
from airunner.utils.convert_base64_to_image import convert_base64_to_image
from airunner.utils.convert_image_to_base64 import convert_image_to_base64
from airunner.widgets.canvas.draggables.draggable_pixmap import DraggablePixmap
from airunner.windows.main.settings_mixin import SettingsMixin


class CustomScene(
    QGraphicsScene,
    MediatorMixin,
    SettingsMixin
):
    def __init__(self, canvas_type: str):
        self.canvas_type = canvas_type
        self.logger = Logger(prefix=self.__class__.__name__)
        MediatorMixin.__init__(self)
        SettingsMixin.__init__(self)
        self.image_backup = None
        self.previewing_filter = False
        self.painter = None
        self.image: QImage = None
        self.item: QGraphicsPixmapItem = None
        super().__init__()
        self.last_export_path = None
        self._target_size = None

        # Add a variable to store the last mouse position
        self.last_pos = None
        self.start_pos = None
        self.selection_start_pos = None
        self.selection_stop_pos = None
        self.do_update = False
        self.generate_image_time_in_ms = 0.5
        self.do_generate_image = False
        self.generate_image_time = 0
        self.undo_history = []
        self.redo_history = []
        self.right_mouse_button_pressed = False
        self.handling_event = False

        for signal, handler in [
            (SignalCode.CANVAS_COPY_IMAGE_SIGNAL, self.on_canvas_copy_image_signal),
            (SignalCode.CANVAS_CUT_IMAGE_SIGNAL, self.on_canvas_cut_image_signal),
            (SignalCode.CANVAS_ROTATE_90_CLOCKWISE_SIGNAL, self.on_canvas_rotate_90_clockwise_signal),
            (SignalCode.CANVAS_ROTATE_90_COUNTER_CLOCKWISE_SIGNAL, self.on_canvas_rotate_90_counter_clockwise_signal),
            (SignalCode.CANVAS_PASTE_IMAGE_SIGNAL, self.on_paste_image_from_clipboard),
            (SignalCode.CANVAS_EXPORT_IMAGE_SIGNAL, self.on_export_image_signal),
            (SignalCode.CANVAS_IMPORT_IMAGE_SIGNAL, self.on_import_image_signal),
            (SignalCode.CANVAS_APPLY_FILTER_SIGNAL, self.on_apply_filter_signal),
            (SignalCode.CANVAS_CANCEL_FILTER_SIGNAL, self.on_cancel_filter_signal),
            (SignalCode.CANVAS_PREVIEW_FILTER_SIGNAL, self.on_preview_filter_signal),
            (SignalCode.CANVAS_LOAD_IMAGE_FROM_PATH_SIGNAL, self.on_load_image_from_path),
            (SignalCode.ENGINE_RESPONSE_WORKER_RESPONSE_SIGNAL, self.on_image_generated_signal),
            (SignalCode.UNDO_SIGNAL, self.on_action_undo_signal),
            (SignalCode.REDO_SIGNAL, self.on_action_redo_signal),
            (SignalCode.HISTORY_CLEAR_SIGNAL, self.clear_history),
            (SignalCode.CANVAS_CLEAR, self.on_canvas_clear_signal),
            (SignalCode.MASK_LAYER_TOGGLED, self.on_mask_layer_toggled),
        ]:
            self.register(signal, handler)

    @property
    def current_tool(self):
        return CanvasToolName(self.application_settings.current_tool)

    @property
    def settings_key(self):
        return self.property("settings_key")

    @property
    def current_settings(self):
        settings = None
        if self.settings_key == "controlnet_settings":
            settings = self.controlnet_settings
        elif self.settings_key == "image_to_image_settings":
            settings = self.image_to_image_settings
        elif self.settings_key == "outpaint_settings":
            settings = self.outpaint_settings
        elif self.settings_key == "drawing_pad_settings":
            settings = self.drawing_pad_settings
        if not settings:
            raise ValueError(f"Settings not found for key: {self.settings_key}")
        return settings

    @property
    def image_pivot_point(self):
        return QPoint(
            self.current_settings.x_pos,
            self.current_settings.y_pos
        )

    @property
    def current_active_image(self) -> Image:
        base_64_image = self.current_settings.image
        try:
            return convert_base64_to_image(base_64_image)
        except PIL.UnidentifiedImageError:
            return None

    @image_pivot_point.setter
    def image_pivot_point(self, value):
        self.emit_signal(SignalCode.LAYER_UPDATE_CURRENT_SIGNAL, {
            "pivot_point_x": value.x(),
            "pivot_point_y": value.y()
        })

    def on_export_image_signal(self):
        image = self.current_active_image
        if image:
            # Set the parent window to the main application window
            parent_window = self.views()[0].window()

            # Use the last export path if available
            initial_dir = self.last_export_path if self.last_export_path else ""

            file_dialog = QFileDialog(parent_window, "Save Image", initial_dir, f"Image Files ({' '.join(VALID_IMAGE_FILES)})")
            file_dialog.setAcceptMode(QFileDialog.AcceptMode.AcceptSave)
            if file_dialog.exec() == QFileDialog.DialogCode.Accepted:
                file_path = file_dialog.selectedFiles()[0]
                if file_path == "":
                    return

                # Update the last export path
                self.last_export_path = os.path.dirname(file_path)

                # If missing file extension, add it
                if not file_path.endswith(VALID_IMAGE_FILES):
                    file_path = f"{file_path}.png"

                export_image(image, file_path)

    def on_import_image_signal(self):
        file_path, _ = QFileDialog.getOpenFileName(
            None,
            "Open Image",
            "",
            f"Image Files ({' '.join(VALID_IMAGE_FILES)})"
        )
        if file_path == "":
            return
        self.on_load_image_signal(file_path)

    def on_paste_image_from_clipboard(self):
        image = self._paste_image_from_clipboard()
        #self.delete_image()

        if self.application_settings.resize_on_paste:
            image = self._resize_image(image)
        image = convert_image_to_base64(image)
        self._update_current_settings("image", image)
        self.refresh_image()

    def on_load_image_from_path(self, message):
        image_path = message["image_path"]
        if image_path is None or image_path == "":
            return
        image = Image.open(image_path)
        self._load_image_from_object(image)

    def on_load_image_signal(self, image_path: str):
        image = self._load_image(image_path)
        if self.application_settings.resize_on_paste:
            image = self._resize_image(image)
        self.initialize_image(image)

    def on_apply_filter_signal(self, message):
        self._apply_filter(message)

    def on_cancel_filter_signal(self):
        image = self._cancel_filter()
        if image:
            self._load_image_from_object(image=image)

    def on_preview_filter_signal(self, message):
        filter_object: ImageFilter.Filter = message["filter_object"]
        filtered_image = self._preview_filter(
            self.current_active_image,
            filter_object
        )
        self._load_image_from_object(image=filtered_image)

    def on_image_generated_signal(self, response):
        code = response["code"]
        if code == EngineResponseCode.IMAGE_GENERATED:
            message = response["message"]
            if message is None:
                self.logger.error("No message received from engine")
                return
            images = message["images"]
            if len(images) == 0:
                self.logger.debug("No images received from engine")
            elif message:
                self._create_image(
                    image=images[0].convert("RGBA"),
                    is_outpaint=message["is_outpaint"],
                    outpaint_box_rect=message["active_rect"]
                )
        else:
            self.logger.error(f"Unhandled response code: {code}")
        self.emit_signal(SignalCode.APPLICATION_STOP_SD_PROGRESS_BAR_SIGNAL)
        callback = response.get("callback", None)
        if callback:
            callback(response)

    def on_canvas_clear_signal(self):
        self._update_current_settings("image", None)
        self.delete_image()

    def on_mask_layer_toggled(self):
        self.initialize_image()

    def on_canvas_copy_image_signal(self):
        self._copy_image(self.current_active_image)

    def on_canvas_cut_image_signal(self):
        self._cut_image(self.current_active_image)

    def on_canvas_rotate_90_clockwise_signal(self):
        print("on_canvas_rotate_90_clockwise_signal")
        self._rotate_90_clockwise()

    def on_canvas_rotate_90_counter_clockwise_signal(self):
        self._rotate_90_counterclockwise()

    def on_action_undo_signal(self):
        if len(self.undo_history) == 0:
            return
        data = self.undo_history.pop()
        self._add_image_to_redo()
        self.history_set_image(data)

    def on_action_redo_signal(self):
        if len(self.redo_history) == 0:
            return
        data = self.redo_history.pop()
        self._add_image_to_undo()
        self.history_set_image(data)

    def showEvent(self, event):
        super().showEvent(event)
        self.initialize_image()

    def dragEnterEvent(self, event: QDragEnterEvent):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()

    def dragMoveEvent(self, event: QDragMoveEvent):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()

    def dropEvent(self, event: QDropEvent):
        for url in event.mimeData().urls():
            path = url.toLocalFile()
            if path.split('.')[-1].lower().encode() in QImageReader.supportedImageFormats():
                self._load_image(path)

    def wheelEvent(self, event):
        if not hasattr(event, "delta"):
            return

        # Check if the Ctrl key is pressed
        if event.modifiers() & Qt.KeyboardModifier.ControlModifier:
            zoom_in_factor = self.grid_settings.zoom_in_step
            zoom_out_factor = -self.grid_settings.zoom_out_step

            if event.delta() > 0:
                zoom_factor = zoom_in_factor
            else:
                zoom_factor = zoom_out_factor

            # Update zoom level
            zoom_level = self.grid_settings.zoom_level
            zoom_level += zoom_factor
            if zoom_level < 0.1:
                zoom_level = 0.1
            self.update_grid_settings("zoom_level", zoom_level)

            self.emit_signal(SignalCode.CANVAS_ZOOM_LEVEL_CHANGED)

    def mousePressEvent(self, event):
        if isinstance(event, QGraphicsSceneMouseEvent):
            if event.button() == Qt.MouseButton.RightButton:
                self.right_mouse_button_pressed = True
                self.start_pos = event.scenePos()
            elif not self._handle_left_mouse_press(event):
                super(CustomScene, self).mousePressEvent(event)
        self._handle_cursor(event)
        self.last_pos = event.scenePos()
        self.update()

        if event.button() == Qt.MouseButton.LeftButton:
            self._handle_left_mouse_press(event)
            self._handle_cursor(event)
            if not self.is_brush_or_eraser:
                super().mousePressEvent(event)
            elif self.drawing_pad_settings.enable_automatic_drawing:
                self.emit_signal(SignalCode.INTERRUPT_IMAGE_GENERATION_SIGNAL)

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.MouseButton.RightButton:
            self.right_mouse_button_pressed = False
        elif not self._handle_left_mouse_release(event):
            super(CustomScene, self).mouseReleaseEvent(event)
        self._handle_cursor(event)

        super().mouseReleaseEvent(event)
        if event.button() == Qt.MouseButton.LeftButton:
            self.last_pos = None
            self.start_pos = None
            self.do_update = False
            if self.drawing_pad_settings.enable_automatic_drawing:
                if self._is_drawing or self._is_erasing:
                    self.emit_signal(SignalCode.SD_GENERATE_IMAGE_SIGNAL)
            self._is_drawing = False
            self._is_erasing = False

    def mouseMoveEvent(self, event):
        if self.right_mouse_button_pressed:
            view = self.views()[0]
            view.setTransformationAnchor(view.ViewportAnchor.NoAnchor)
            view.setResizeAnchor(view.ViewportAnchor.NoAnchor)
            delta = event.scenePos() - self.last_pos
            scale_factor = view.transform().m11()  # Get the current scale factor
            view.translate(delta.x() / scale_factor, delta.y() / scale_factor)
            self.last_pos = event.scenePos()
        else:
            self._handle_cursor(event)
            super(CustomScene, self).mouseMoveEvent(event)

        self.last_pos = event.scenePos()
        self.update()

    def event(self, event):
        if self.handling_event:
            return False  # Prevent recursive event calls

        self.handling_event = True
        try:
            if type(event) == QEnterEvent:
                self._handle_cursor(event)
            return super(CustomScene, self).event(event)
        finally:
            self.handling_event = False

    def leaveEvent(self, event):
        self._handle_cursor(event)
        super(CustomScene, self).leaveEvent(event)

    def refresh_image(self, image: Image = None):
        image = image if image is not None else self.current_active_image

        # Save the current viewport position
        view = self.views()[0]
        current_viewport_rect = view.mapToScene(view.viewport().rect()).boundingRect()

        # Update the pixmap item, image+painter and scene
        try:
            item_scene = self.item.scene()
        except AttributeError:
            item_scene = None
        if item_scene is not None:
            item_scene.removeItem(self.item)
        if self.painter and self.painter.isActive():
            self.painter.end()
        self.initialize_image(image)
        # Restore the viewport position
        view.setSceneRect(current_viewport_rect)

        # Restore the viewport position
        view.setSceneRect(current_viewport_rect)

    def history_set_image(self, data: dict):
        if data is not None:
            self.refresh_image(data["image"])

    def clear_history(self):
        self.__clear()

    def delete_image(self):
        self.logger.debug("Deleting image from canvas")

        item_scene = self.item.scene()
        if item_scene is not None:
            item_scene.removeItem(self.item)

        if self.painter and self.painter.isActive():
            self.painter.end()
        self._update_current_settings("image", None)
        self.image = None
        self.initialize_image()

    def set_image(self, pil_image: Image = None):
        base64image = None
        if not pil_image:
            base64image = self.current_settings.image

        if base64image is not None:
            try:
                pil_image = convert_base64_to_image(base64image).convert("RGBA")
            except AttributeError:
                self.logger.warning("Failed to convert base64 to image")
            except PIL.UnidentifiedImageError:
                pil_image = None

        if pil_image is not None:
            try:
                img = ImageQt.ImageQt(pil_image)
            except AttributeError as _e:
                img = None
            except IsADirectoryError:
                img = None
            self.image = img
        else:
            self.image = QImage(
                self.application_settings.working_width,
                self.application_settings.working_height,
                QImage.Format.Format_ARGB32
            )
            self.image.fill(Qt.GlobalColor.transparent)

    def set_item(self, image: QImage = None, z_index: int = 1):
        self.setSceneRect(0, 0, 512, 512)
        if image is not None:
            pixmap = QPixmap.fromImage(image)
            if self.item is None:
                self.item = DraggablePixmap(pixmap)
            else:
                self.item.setPixmap(pixmap)
            self.item.setZValue(z_index)
            if self.item.scene() is None:
                self.addItem(self.item)

    def clear_selection(self):
        self.selection_start_pos = None
        self.selection_stop_pos = None

    def initialize_image(self, image: Image = None):
        self.stop_painter()
        self.set_image(image)
        self.set_item(self.image)
        self.set_painter(self.image)

    def stop_painter(self):
        if self.painter is not None and self.painter.isActive():
            self.painter.end()

    def set_painter(self, image: QImage):
        if image is None:
            return
        try:
            self.painter = QPainter(image)
        except TypeError as _e:
            self.logger.error("Failed to initialize painter in initialize_image")

    def _update_current_settings(self, key, value):
        if self.settings_key == "controlnet_settings":
            self.update_controlnet_settings(key, value)
        elif self.settings_key == "image_to_image_settings":
            self.update_image_to_image_settings(key, value)
        elif self.settings_key == "outpaint_settings":
            self.update_outpaint_settings(key, value)
        elif self.settings_key == "drawing_pad_settings":
            self.update_drawing_pad_settings(key, value)

    def _load_image_from_object(
        self,
        image: Image,
        is_outpaint: bool = False
    ):
        self._add_image_to_scene(
            is_outpaint=is_outpaint,
            image=image
        )

    def _paste_image_from_clipboard(self):
        self.logger.debug("paste image from clipboard")
        image = self._get_image_from_clipboard()

        if not image:
            self.logger.debug("No image in clipboard")
            return
        return image

    def _get_image_from_clipboard(self):
        if platform_info.is_windows():
            return self._image_from_system_clipboard_windows()
        return self._image_from_system_clipboard_linux()

    def _image_from_system_clipboard_windows(self):
        self.logger.debug("image_from_system_clipboard_windows")
        import win32clipboard
        try:
            win32clipboard.OpenClipboard()
            data = win32clipboard.GetClipboardData(win32clipboard.CF_DIB)
            win32clipboard.CloseClipboard()
            # convert bytes to image
            image = Image.open(io.BytesIO(data))
            return image
        except Exception as e:
            print(e)
            return None

    def _image_from_system_clipboard_linux(self):
        self.logger.debug("image_from_system_clipboard_linux")
        try:
            image = ImageGrab.grabclipboard()
            if not image:
                self.logger.debug("No image in clipboard")
                return None
            # with transparency
            image = image.convert("RGBA")
            return image
        except Exception as e:
            print(e)
            return None

    def _copy_image(
        self,
        image: Image
    ) -> DraggablePixmap:
        return self._move_pixmap_to_clipboard(image)

    def _move_pixmap_to_clipboard(self, image: Image) -> Image:
        if platform_info.is_windows():
            return self._image_to_system_clipboard_windows(image)
        return self._image_to_system_clipboard_linux(image)

    def _image_to_system_clipboard_windows(self, image: Image) -> Image:
        if image is None:
            return None
        self.logger.debug("image_to_system_clipboard_windows")
        import win32clipboard
        data = io.BytesIO()
        image.save(data, format="png")
        data = data.getvalue()
        win32clipboard.OpenClipboard()
        win32clipboard.EmptyClipboard()
        win32clipboard.SetClipboardData(win32clipboard.CF_DIB, data)
        win32clipboard.CloseClipboard()
        return image

    def _image_to_system_clipboard_linux(self, image: Image) -> Image:
        if image is None:
            return None
        data = io.BytesIO()

        # Save PIL Image to BytesIO
        image.save(data, format="png")

        data = data.getvalue()
        try:
            subprocess.Popen(["xclip", "-selection", "clipboard", "-t", "image/png"],
                             stdin=subprocess.PIPE).communicate(data)
        except FileNotFoundError:
            self.logger.error("xclip not found. Please install xclip to copy image to clipboard.")
        return image

    def _create_image(self, image, is_outpaint, outpaint_box_rect):
        if self.application_settings.resize_on_paste:
            image = self._resize_image(image)
        if image is not None:
            self._add_image_to_scene(
                image,
                is_outpaint=is_outpaint,
                outpaint_box_rect=outpaint_box_rect
            )

    def _resize_image(self, image: Image) -> Image:
        if image is None:
            return

        max_size = (
            self.application_settings.working_width,
            self.application_settings.working_height
        )
        image.thumbnail(max_size, PIL.Image.Resampling.BICUBIC)
        return image

    def _add_image_to_scene(
        self,
        image: Image,
        is_outpaint: bool = False,
        outpaint_box_rect: QPoint = None,
        border_size: int = 1,  # size of the border
        border_color: tuple = (255, 0, 0, 255)  # color of the border in RGBA format
    ):
        """
        Adds a given image to the scene
        :param image: Image object to add to the scene
        :param is_outpaint: bool indicating if the image is an outpaint
        :param outpaint_box_rect: QPoint indicating the root point of the image
        :param border_size: int indicating the size of the border
        :param border_color: tuple indicating the color of the border
        :return:
        """
        # image = ImageOps.expand(image, border=border_size, fill=border_color)

        if image is None:
            self.logger.warning("Image is None, unable to add to scene")
            return

        if is_outpaint:
            image, root_point, pivot_point = self._handle_outpaint(
                outpaint_box_rect,
                image,
                action=GeneratorSection.OUTPAINT.value
            )
        self._set_current_active_image(image)
        base64_image = convert_image_to_base64(image)
        self._update_current_settings("image", base64_image)
        q_image = ImageQt.ImageQt(image)
        self.item.setPixmap(QPixmap.fromImage(q_image))
        self.item.setZValue(0)
        self.update()

        # Add the mask to the scene
        self.set_mask()

    def _handle_outpaint(self, outpaint_box_rect, outpainted_image, action=None) -> [Image, QPoint, QPoint]:
        if self.current_active_image is None:
            point = QPoint(outpaint_box_rect.x(), outpaint_box_rect.y())
            return outpainted_image, QPoint(0, 0), point

        # make a copy of the current canvas image
        existing_image_copy = self.current_active_image.copy()
        width = existing_image_copy.width
        height = existing_image_copy.height

        mask_image = self.drawing_pad_mask

        pivot_point = self.image_pivot_point
        root_point = QPoint(0, 0)
        current_image_position = QPoint(0, 0)

        is_drawing_left = outpaint_box_rect.x() < current_image_position.x()
        is_drawing_right = outpaint_box_rect.x() > current_image_position.x()
        is_drawing_up = outpaint_box_rect.y() < current_image_position.y()
        is_drawing_down = outpaint_box_rect.y() > current_image_position.y()

        if is_drawing_down:
            height += outpaint_box_rect.y()
        if is_drawing_right:
            width += outpaint_box_rect.x()
        if is_drawing_up:
            height += current_image_position.y()
            root_point.setY(outpaint_box_rect.y())
        if is_drawing_left:
            width += current_image_position.x()
            root_point.setX(outpaint_box_rect.x())

        new_dimensions = (width, height)

        new_image = Image.new("RGBA", new_dimensions, (0, 0, 0, 0))
        new_image_a = Image.new("RGBA", new_dimensions, (0, 0, 0, 0))
        new_image_b = Image.new("RGBA", new_dimensions, (0, 0, 0, 0))

        image_root_point = QPoint(root_point.x(), root_point.y())
        image_pivot_point = QPoint(pivot_point.x(), pivot_point.y())

        new_image_a.paste(outpainted_image, (int(outpaint_box_rect.x()), int(outpaint_box_rect.y())))
        new_image_b.paste(existing_image_copy, (current_image_position.x(), current_image_position.y()))

        # Convert mask to binary mask
        mask = mask_image.convert("L").point(lambda p: p > 128 and 255)
        inverted_mask = Image.eval(mask, lambda p: 255 - p)
        new_image_b = Image.composite(new_image_b, Image.new("RGBA", new_image_b.size), inverted_mask)

        new_image = Image.alpha_composite(new_image, new_image_a)
        new_image = Image.alpha_composite(new_image, new_image_b)

        return new_image, image_root_point, image_pivot_point

    def _set_current_active_image(self, image: Image):
        self.logger.debug("Setting current active image")
        self.refresh_image(image)

    def _rotate_90_clockwise(self):
        self.rotate_image(-90)

    def _rotate_90_counterclockwise(self):
        self.rotate_image(90)

    def rotate_image(
        self,
        angle: float
    ):
        image = self.current_active_image
        if image is not None:
            image = image.rotate(angle, expand=True)
            self._update_current_settings("image", convert_image_to_base64(image))
            self.initialize_image(image)

    def _add_undo_history(self, data: dict):
        self.undo_history.append(data)

    def _add_redo_history(self, data: dict):
        self.redo_history.append(data)

    def _clear_history(self):
        self.undo_history = []
        self.redo_history = []

    def _cut_image(self, image: Image = None) -> Image:
        image = self._copy_image(image)
        if image is not None:
            self._add_image_to_undo()
            self.delete_image()

    def _add_image_to_undo(self):
        image = self.current_active_image
        if image is not None:
            self._add_undo_history({
                "image": image.copy()
            })

    def _add_image_to_redo(self):
        image = self.current_active_image
        if image is not None:
            self._add_redo_history({
                "image": image.copy()
            })

    def _handle_mouse_event(self, event, is_press_event) -> bool:
        if isinstance(event, QMouseEvent) and event.button() == Qt.MouseButton.LeftButton:
            view = self.views()[0]
            pos = view.mapFromScene(event.scenePos())
            if (
                self.grid_settings.snap_to_grid and
                self.current_tool is CanvasToolName.SELECTION
            ):
                x, y = snap_to_grid(self.grid_settings, pos.x(), pos.y(), False)
                pos = QPoint(x, y)
                if is_press_event:
                    self.selection_stop_pos = None
                    self.selection_start_pos = QPoint(pos.x(), pos.y())
                else:
                    self.selection_stop_pos = QPoint(pos.x(), pos.y())
                self.emit_signal(SignalCode.CANVAS_DO_DRAW_SELECTION_AREA_SIGNAL)
                return True
        return False

    def _handle_left_mouse_press(self, event) -> bool:
        try:
            self.start_pos = event.scenePos()
        except AttributeError:
            self.logger.error("Failed to get scenePos from left click event")
        return self._handle_mouse_event(event, True)

    def _handle_left_mouse_release(self, event) -> bool:
        return self._handle_mouse_event(event, False)

    def _handle_cursor(self, event):
        self.emit_signal(
            SignalCode.CANVAS_UPDATE_CURSOR,
            {
                "event": event
            }
        )

    def _load_image(self, image_path: str) -> Image:
        image = Image.open(image_path)
        return image

    def _apply_filter(self, _filter_object: ImageFilter.Filter):
        self.previewing_filter = False
        self.image_backup = None

    def _cancel_filter(self) -> Image:
        image = None
        if self.image_backup:
            image = self.image_backup.copy()
            self.image_backup = None
        self.previewing_filter = False
        return image

    def _preview_filter(
        self,
        image: Image,
        filter_object: ImageFilter.Filter
    ):
        if not image:
            return
        if not self.previewing_filter:
            self.image_backup = image.copy()
            self.previewing_filter = True
        else:
            image = self.image_backup.copy()
        filtered_image = filter_object.filter(image)
        return filtered_image



import os
from urllib.parse import urlparse

from airunner.aihandler.models.settings_models import Lora, AIModels
from airunner.enums import SignalCode
from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.model_manager.templates.import_ui import Ui_import_model_widget
from airunner.aihandler.stablediffusion.download_civitai import DownloadCivitAI
from airunner.aihandler.stablediffusion.download_huggingface import DownloadHuggingface
from airunner.windows.main.ai_model_mixin import AIModelMixin
from airunner.windows.main.pipeline_mixin import PipelineMixin


class ImportWidget(
    BaseWidget,
    PipelineMixin,
    AIModelMixin
):
    widget_class_ = Ui_import_model_widget
    model_widgets = []

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        AIModelMixin.__init__(self)
        self.current_model_data = None
        self.is_civitai = False
        self.register(SignalCode.DOWNLOAD_COMPLETE, self.show_download_complete)
        self.show_import_form()
        self.download_civit_ai = DownloadCivitAI()
        self.download_huggingface = DownloadHuggingface()

    def action_clicked_button_import(self):
        self.show_model_select_form()

    def action_clicked_button_download(self):
        self.show_download_form()

    def action_clicked_button_cancel(self):
        if self.is_civitai:
            self.download_civit_ai.stop_download()
        else:
            self.download_huggingface.stop_download()
        self.show_import_form()

    def action_download_complete_continue(self):
        self.show_import_form()

    def show_import_form(self):
        self.ui.import_form.show()
        self.ui.model_select_form.hide()
        self.ui.download_form.hide()
        self.ui.download_complete_form.hide()

    def show_model_select_form(self):
        self.ui.import_form.hide()
        self.ui.model_select_form.show()
        self.ui.download_form.hide()
        self.ui.download_complete_form.hide()
        self.import_models()

    def show_download_form(self):
        self.ui.import_form.hide()
        self.ui.model_select_form.hide()
        self.ui.download_form.show()
        self.ui.download_complete_form.hide()
        self.download_model()

    def show_download_complete(self, data=None):
        self.ui.import_form.hide()
        self.ui.model_select_form.hide()
        self.ui.download_form.hide()
        self.ui.download_complete_form.show()
    
    def download_model(self):
        # get value from import_widget.model_choices
        download_url, file, model_version = self.ui.model_choices.currentData()
        size_kb = file["sizeKB"]

        model_data = self.current_model_data
        name = model_data["name"] + " " + model_version["name"]
        
        model_version_name = model_version["name"]
        category = "stablediffusion"
        pipeline_action = "txt2img"
        if "inpaint" in model_version_name:
            pipeline_action = "outpaint"
        diffuser_model_version = model_version["baseModel"]
        model_type = model_data["type"]
        file_path = self.download_path(
            file,
            diffuser_model_version,
            pipeline_action,
            model_type
        )

        trained_words = model_version.get("trainedWords", [])
        if isinstance(trained_words, str):
            trained_words = [trained_words]
        trained_words = ",".join(trained_words)
        if model_type == "Checkpoint":
            model = AIModels()
            model.name = name
            model.path = file_path
            model.branch = "main"
            model.version = diffuser_model_version
            model.category = category
            model.pipeline_action = pipeline_action
            model.enabled = True
            model.model_type = "art"
            model.is_default = False
            self.emit_signal(SignalCode.AI_MODELS_SAVE_OR_UPDATE_SIGNAL, {"models": [model]})
        elif model_type == "LORA":
            name = file["name"].replace(".ckpt", "").replace(".safetensors", "").replace(".pt", "")
            new_lora = Lora(
                name=name,
                path=file_path,
                scale=1,
                enabled=True,
                loaded=False,
                trigger_word=trained_words,
                version=model_version["baseModel"]
            )
            self.create_lora(new_lora)
        elif model_type == "TextualInversion":
            # name = file_path.split("/")[-1].split(".")[0]
            # embedding_exists = session.query(Embedding).filter_by(
            #     name=name,
            #     path=file_path,
            # ).first()
            # if not embedding_exists:
            #     new_embedding = Embedding(
            #         name=name,
            #         path=file_path,
            #         active=True,
            #         tags=trained_words,
            #     )
            #     session.add(new_embedding)
            # TODO: handle textual inversion
            pass
        elif model_type == "VAE":
            # todo save vae here
            pass
        elif model_type == "Controlnet":
            # todo save controlnet here
            pass
        elif model_type == "Poses":
            # todo save poses here
            pass
        
        self.logger.debug("Starting download")
        self.ui.downloading_label.setText(f"Downloading {name}")
        self.download_model_thread(download_url, file_path, size_kb)
        # self.thread = threading.Thread(target=self.download_model_thread, args=(download_url, file_path, size_kb))
        # self.thread.start()

    def download_model_thread(self, download_url, file_path, size_kb):
        if self.is_civitai:
            self.download_civit_ai.download_model(download_url, file_path, size_kb, self.download_callback)
        else:
            self.download_huggingface.download_model(download_url, callback=self.download_callback)

    def download_callback(self, current_size, total_size):
        current_size = int(current_size / total_size * 100)
        self.ui.download_progress_bar.setValue(current_size)
        if current_size >= total_size:
            self.reset_form()
            self.emit_signal(SignalCode.AI_MODELS_CREATE_SIGNAL)
            self.show_items_in_scrollarea()

    def reset_form(self):
        self.ui.download_progress_bar.setValue(0)
        self.ui.download_progress_bar.hide()
        self.ui.downloading_label.hide()
        self.ui.cancel_download_button.hide()

    def show_items_in_scrollarea(self):
        self.ui.import_form.show()
        self.ui.model_select_form.hide()
        self.ui.download_form.hide()
        self.ui.download_complete_form.hide()
        self.import_models()

    def parse_url(self) -> str:
        url = self.ui.import_url.text()
        model_id = None

        try:
            model_id = int(url.split("models/")[1])
        except Exception as e:
            print(f"Failed to parse model id from url: {url}")
            print(e)

        if model_id is None:
            try:
                print("setting model id")
                model_id = int(url.split("models/")[1].split("/")[0])
                print("model id set to ", model_id)
            except Exception as e:
                print(f"2 Failed to parse model id from url: {url}")
                print(e)

        parsed_url = urlparse(url)
        self.is_civitai = "civitai.com" in parsed_url.netloc
        return str(model_id)

    def import_models(self):
        data = None
        model_id = self.parse_url()

        if model_id:
            data = DownloadCivitAI().get_json(model_id=model_id)

        self.current_model_data = data

        model_name = ""
        model_versions = []
        if data:
            if "name" in data:
                model_name = data["name"]
                model_versions = data["modelVersions"]

        self.ui.model_choices.clear()
        for model_version in model_versions:
            version_name = model_version["name"]
            download_url = model_version["downloadUrl"]
            sd_model_version = model_version["baseModel"]
            files = model_version["files"]
            # try:
            #     url = file["url"]
            # except KeyError:
            #     url = ""
            download_choice = f"{model_name} - {version_name} - {sd_model_version}"
            for file in files:
                self.ui.model_choices.addItem(
                    download_choice,
                    (download_url, file, model_version)
                )

        self.ui.model_choices.currentIndexChanged.connect(self.model_version_changed)

        if data:
            self.ui.name.setText(data["name"])
            self.ui.name.show()
            if not data["nsfw"]:
                self.ui.nsfw_label.hide()
            else:
                self.ui.nsfw_label.show()

            if "creator" in data and "username" in data["creator"]:
                self.ui.creator.setText(
                    f"By {data['creator']['username']}"
                )
            else:
                self.ui.creator.show()
                self.ui.creator.setText("")
        else:
            self.ui.name.hide()
            self.ui.nsfw_label.hide()
            self.ui.creator.hide()

        self.set_model_form_data()
    
    def model_version_changed(self, index):
        self.set_model_form_data()
    
    def download_path(self, file, version, pipeline_action, model_type):
        base_path = self.path_settings.base_path
        if model_type == "LORA":
            action = "lora"
        elif model_type == "Checkpoint":
            action = pipeline_action
            if action == "img2img":
                action = "txt2img"
        elif model_type == "TextualInversion":
            action = "embeddings"
        elif model_type == "VAE":
            action = "vae"
        elif model_type == "Controlnet":
            action = "controlnet"
        return os.path.expanduser(
            os.path.join(
                base_path,
                "art/models",
                version,
                action,
                file["name"]
            )
        )

    def get_pipeline_classname(self, pipeline_action, version, category):
        pipelines = self.get_pipelines(pipeline_action, version, category)
        if len(pipelines) > 0:
            return pipelines[0]["classname"]
        else:
            return None

    def set_model_form_data(self):
        try:
            download_url, file, model_version = self.ui.model_choices.currentData()
        except TypeError:
            return
        model_version_name = model_version["name"]

        categories = self.ai_model_categories()
        actions = [pipeline.pipeline_action for pipeline in self.pipelines]
        category = "stablediffusion"
        pipeline_action = "txt2img"
        if "inpaint" in model_version_name:
            pipeline_action = "outpaint"
        diffuser_model_version = model_version["baseModel"]
        pipeline_class = self.get_pipeline_classname(pipeline_action, diffuser_model_version, category)
        diffuser_model_versions = self.ai_model_versions()
        path = self.download_path(
            file,
            diffuser_model_version,
            pipeline_action,
            self.current_model_data["type"]
        )

        self.ui.model_form.set_model_form_data(
            categories, 
            actions, 
            diffuser_model_versions, 
            category, 
            pipeline_action, 
            pipeline_class, 
            diffuser_model_version, 
            path, 
            self.current_model_data["name"],
            model_data=self.current_model_data,
            model_type=self.current_model_data["type"]
        )

        if self.is_civitai:
            self.ui.model_form.ui.branch_label.hide()
            self.ui.model_form.ui.branch_line_edit.hide()



from PySide6.QtWidgets import (
    QVBoxLayout,
    QLabel,
    QSpinBox,
    QPushButton
)
from PySide6.QtGui import QPixmap
from threading import Lock
from airunner.enums import SignalCode
from airunner.widgets.base_widget import BaseWidget


class BlenderViewportWidget(BaseWidget):
    def __init__(self):
        super().__init__()
        self.setup_ui()
        self.image_lock = Lock()
        self.camera_lock = Lock()  # Lock for controlling camera size

        self.register(SignalCode.VIEWPORT_IMAGE, self.update_viewport_image)

    def setup_ui(self):
        layout = QVBoxLayout()
        self.image_label = QLabel()
        layout.addWidget(self.image_label)

        # Add UI elements for changing viewport size
        self.size_spinbox = QSpinBox()
        self.size_spinbox.setRange(1, 1000)  # Adjust range as needed
        layout.addWidget(self.size_spinbox)

        # Button to apply viewport size change
        self.apply_button = QPushButton("Apply Size Change")
        self.apply_button.clicked.connect(self.apply_size_change)
        layout.addWidget(self.apply_button)

        self.setLayout(layout)

    def apply_size_change(self):
        new_size = self.size_spinbox.value()
        with self.camera_lock:  # Acquire lock before making API call to Blender
            # Call function to update Blender camera size
            self.update_blender_camera_size(new_size)

    def update_viewport_image(self, image):
        with self.image_lock:
            pixmap = QPixmap.fromImage(image)
            self.image_label.setPixmap(pixmap)

    def update_blender_camera_size(self, new_size):
        # Your code to update Blender camera size through an API call
        # This could involve sending a message to Blender or invoking a script
        pass



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'model.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QGridLayout, QHBoxLayout,
    QHeaderView, QPushButton, QSizePolicy, QSpacerItem,
    QTableWidget, QTableWidgetItem, QVBoxLayout, QWidget)
import airunner.resources_light_rc
import airunner.resources_dark_rc

class Ui_model_widget(object):
    def setupUi(self, model_widget):
        if not model_widget.objectName():
            model_widget.setObjectName(u"model_widget")
        model_widget.resize(395, 295)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(model_widget.sizePolicy().hasHeightForWidth())
        model_widget.setSizePolicy(sizePolicy)
        model_widget.setMinimumSize(QSize(0, 0))
        self.gridLayout = QGridLayout(model_widget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.name = QCheckBox(model_widget)
        self.name.setObjectName(u"name")
        font = QFont()
        font.setPointSize(8)
        font.setBold(True)
        self.name.setFont(font)

        self.horizontalLayout_2.addWidget(self.name)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.toolButton = QPushButton(model_widget)
        self.toolButton.setObjectName(u"toolButton")
        self.toolButton.setMinimumSize(QSize(24, 24))
        self.toolButton.setMaximumSize(QSize(24, 24))
        self.toolButton.setCursor(QCursor(Qt.PointingHandCursor))
        icon = QIcon()
        icon.addFile(u":/icons/light/eye-look-icon.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.toolButton.setIcon(icon)
        self.toolButton.setCheckable(True)
        self.toolButton.setChecked(False)

        self.horizontalLayout.addWidget(self.toolButton)

        self.delete_button = QPushButton(model_widget)
        self.delete_button.setObjectName(u"delete_button")
        self.delete_button.setMinimumSize(QSize(24, 24))
        self.delete_button.setMaximumSize(QSize(24, 24))
        self.delete_button.setCursor(QCursor(Qt.PointingHandCursor))
        icon1 = QIcon()
        icon1.addFile(u":/icons/light/recycle-bin-line-icon.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.delete_button.setIcon(icon1)

        self.horizontalLayout.addWidget(self.delete_button)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)


        self.verticalLayout.addLayout(self.horizontalLayout)


        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.details = QTableWidget(model_widget)
        if (self.details.columnCount() < 2):
            self.details.setColumnCount(2)
        if (self.details.rowCount() < 7):
            self.details.setRowCount(7)
        __qtablewidgetitem = QTableWidgetItem()
        self.details.setItem(0, 0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.details.setItem(0, 1, __qtablewidgetitem1)
        __qtablewidgetitem2 = QTableWidgetItem()
        self.details.setItem(1, 0, __qtablewidgetitem2)
        __qtablewidgetitem3 = QTableWidgetItem()
        self.details.setItem(1, 1, __qtablewidgetitem3)
        __qtablewidgetitem4 = QTableWidgetItem()
        self.details.setItem(2, 0, __qtablewidgetitem4)
        __qtablewidgetitem5 = QTableWidgetItem()
        self.details.setItem(2, 1, __qtablewidgetitem5)
        __qtablewidgetitem6 = QTableWidgetItem()
        self.details.setItem(3, 0, __qtablewidgetitem6)
        __qtablewidgetitem7 = QTableWidgetItem()
        self.details.setItem(3, 1, __qtablewidgetitem7)
        __qtablewidgetitem8 = QTableWidgetItem()
        self.details.setItem(4, 0, __qtablewidgetitem8)
        __qtablewidgetitem9 = QTableWidgetItem()
        self.details.setItem(4, 1, __qtablewidgetitem9)
        __qtablewidgetitem10 = QTableWidgetItem()
        self.details.setItem(5, 0, __qtablewidgetitem10)
        __qtablewidgetitem11 = QTableWidgetItem()
        self.details.setItem(5, 1, __qtablewidgetitem11)
        __qtablewidgetitem12 = QTableWidgetItem()
        self.details.setItem(6, 0, __qtablewidgetitem12)
        __qtablewidgetitem13 = QTableWidgetItem()
        self.details.setItem(6, 1, __qtablewidgetitem13)
        self.details.setObjectName(u"details")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.MinimumExpanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.details.sizePolicy().hasHeightForWidth())
        self.details.setSizePolicy(sizePolicy1)
        self.details.setMinimumSize(QSize(0, 214))
        self.details.setRowCount(7)
        self.details.setColumnCount(2)
        self.details.horizontalHeader().setVisible(False)
        self.details.horizontalHeader().setStretchLastSection(True)
        self.details.verticalHeader().setVisible(False)
        self.details.verticalHeader().setStretchLastSection(False)

        self.gridLayout.addWidget(self.details, 1, 0, 1, 1)


        self.retranslateUi(model_widget)
        self.toolButton.toggled.connect(model_widget.action_toggled_button_details)
        self.delete_button.clicked.connect(model_widget.action_clicked_button_delete)
        self.details.cellChanged.connect(model_widget.action_cell_changed)

        QMetaObject.connectSlotsByName(model_widget)
    # setupUi

    def retranslateUi(self, model_widget):
        model_widget.setWindowTitle(QCoreApplication.translate("model_widget", u"Form", None))
        model_widget.setProperty("branch", "")
        model_widget.setProperty("path", "")
        model_widget.setProperty("version", "")
        model_widget.setProperty("category", "")
        model_widget.setProperty("pipeline_action", "")
        model_widget.setProperty("pipeline_class", "")
        model_widget.setProperty("prompts", "")
        self.name.setText(QCoreApplication.translate("model_widget", u"UPC", None))
#if QT_CONFIG(tooltip)
        self.toolButton.setToolTip(QCoreApplication.translate("model_widget", u"View details", None))
#endif // QT_CONFIG(tooltip)
        self.toolButton.setText("")
#if QT_CONFIG(tooltip)
        self.delete_button.setToolTip(QCoreApplication.translate("model_widget", u"Delete model", None))
#endif // QT_CONFIG(tooltip)
        self.delete_button.setText("")

        __sortingEnabled = self.details.isSortingEnabled()
        self.details.setSortingEnabled(False)
        ___qtablewidgetitem = self.details.item(0, 0)
        ___qtablewidgetitem.setText(QCoreApplication.translate("model_widget", u"path", None));
        ___qtablewidgetitem1 = self.details.item(0, 1)
        ___qtablewidgetitem1.setText(QCoreApplication.translate("model_widget", u"~/.local/share/airunner/models/model_type", None));
        ___qtablewidgetitem2 = self.details.item(1, 0)
        ___qtablewidgetitem2.setText(QCoreApplication.translate("model_widget", u"branch", None));
        ___qtablewidgetitem3 = self.details.item(1, 1)
        ___qtablewidgetitem3.setText(QCoreApplication.translate("model_widget", u"main", None));
        ___qtablewidgetitem4 = self.details.item(2, 0)
        ___qtablewidgetitem4.setText(QCoreApplication.translate("model_widget", u"version", None));
        ___qtablewidgetitem5 = self.details.item(2, 1)
        ___qtablewidgetitem5.setText(QCoreApplication.translate("model_widget", u"v1", None));
        ___qtablewidgetitem6 = self.details.item(3, 0)
        ___qtablewidgetitem6.setText(QCoreApplication.translate("model_widget", u"category", None));
        ___qtablewidgetitem7 = self.details.item(3, 1)
        ___qtablewidgetitem7.setText(QCoreApplication.translate("model_widget", u"stablediffusion", None));
        ___qtablewidgetitem8 = self.details.item(4, 0)
        ___qtablewidgetitem8.setText(QCoreApplication.translate("model_widget", u"pipeline action", None));
        ___qtablewidgetitem9 = self.details.item(4, 1)
        ___qtablewidgetitem9.setText(QCoreApplication.translate("model_widget", u"txt2img", None));
        ___qtablewidgetitem10 = self.details.item(5, 0)
        ___qtablewidgetitem10.setText(QCoreApplication.translate("model_widget", u"pipeline class", None));
        ___qtablewidgetitem11 = self.details.item(5, 1)
        ___qtablewidgetitem11.setText(QCoreApplication.translate("model_widget", u"diffusers.StableDiffusion", None));
        ___qtablewidgetitem12 = self.details.item(6, 0)
        ___qtablewidgetitem12.setText(QCoreApplication.translate("model_widget", u"prompts", None));
        ___qtablewidgetitem13 = self.details.item(6, 1)
        ___qtablewidgetitem13.setText(QCoreApplication.translate("model_widget", u"prompt", None));
        self.details.setSortingEnabled(__sortingEnabled)

    # retranslateUi




class FilterData:
    class_name = None
    filter_object = None
    options = {}

    def __init__(self, class_name, options):
        self.class_name = class_name
        self.options = options
        self.filter_object = class_name(**options)

    def apply_filter(self, image):
        return self.filter_object.filter(image)


class AutomaticFilterManager:
    registered_filters = []

    def register_filter(self, class_name, **options):
        self.registered_filters.append(FilterData(class_name, options))

    def apply_filters(self, images: list):
        return list(map(self.apply_filters_to_image, images))

    def apply_filters_to_image(self, image):
        for filter_data in self.registered_filters:
            image = filter_data.apply_filter(image)
        return image



from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.tts.templates.tts_preferences_ui import Ui_tts_preferences


class TTSPreferencesWidget(BaseWidget):
    widget_class_ = Ui_tts_preferences

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def initialize_form(self):
        elements = [
            self.ui.enable_tts,
            self.ui.model_combobox,
        ]

        for element in elements:
            element.blockSignals(True)

        tts_model = self.tts_settings.model
        self.ui.enable_tts.setChecked(self.application_settings.tts_enabled)
        self.ui.model_combobox.clear()
        models = ["SpeechT5", "Espeak"]
        self.ui.model_combobox.addItems(models)
        self.ui.model_combobox.setCurrentText(tts_model)
        self._set_model_settings(tts_model)

        for element in elements:
            element.blockSignals(False)

    def enable_tts_changed(self, val):
        self.update_tts_settings("tts_enabled", val)

    def model_changed(self, val):
        self.update_tts_settings("model", val)
        self._set_model_settings(val)

    def _set_model_settings(self, tts_model):
        self.ui.speecht5_preferences.setVisible(tts_model == "SpeechT5")
        self.ui.espeak_preferences.setVisible(tts_model == "Espeak")



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'generatorform.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QFrame, QGridLayout,
    QGroupBox, QHBoxLayout, QLabel, QLineEdit,
    QPlainTextEdit, QProgressBar, QPushButton, QScrollArea,
    QSizePolicy, QSpacerItem, QSplitter, QTabWidget,
    QVBoxLayout, QWidget)

from airunner.widgets.llm.chat_prompt_widget import ChatPromptWidget
from airunner.widgets.llm.llm_history_widget import LLMHistoryWidget
import airunner.resources_light_rc
import airunner.resources_dark_rc

class Ui_generator_form(object):
    def setupUi(self, generator_form):
        if not generator_form.objectName():
            generator_form.setObjectName(u"generator_form")
        generator_form.resize(620, 946)
        font = QFont()
        font.setPointSize(8)
        generator_form.setFont(font)
        generator_form.setCursor(QCursor(Qt.PointingHandCursor))
        self.gridLayout_4 = QGridLayout(generator_form)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.generator_form_tabs = QTabWidget(generator_form)
        self.generator_form_tabs.setObjectName(u"generator_form_tabs")
        self.tab = QWidget()
        self.tab.setObjectName(u"tab")
        self.gridLayout_3 = QGridLayout(self.tab)
        self.gridLayout_3.setSpacing(0)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.stable_diffusion_generator_form = QScrollArea(self.tab)
        self.stable_diffusion_generator_form.setObjectName(u"stable_diffusion_generator_form")
        self.stable_diffusion_generator_form.setFrameShape(QFrame.Shape.NoFrame)
        self.stable_diffusion_generator_form.setFrameShadow(QFrame.Shadow.Plain)
        self.stable_diffusion_generator_form.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 616, 870))
        self.gridLayout = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setHorizontalSpacing(0)
        self.gridLayout.setVerticalSpacing(10)
        self.gridLayout.setContentsMargins(10, 0, 10, 0)
        self.croops_coord_top_left_groupbox = QGroupBox(self.scrollAreaWidgetContents)
        self.croops_coord_top_left_groupbox.setObjectName(u"croops_coord_top_left_groupbox")
        self.horizontalLayout_2 = QHBoxLayout(self.croops_coord_top_left_groupbox)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(2, 2, 2, 2)
        self.crops_coord_top_left_x = QLineEdit(self.croops_coord_top_left_groupbox)
        self.crops_coord_top_left_x.setObjectName(u"crops_coord_top_left_x")

        self.horizontalLayout_2.addWidget(self.crops_coord_top_left_x)

        self.crops_coord_top_left_y = QLineEdit(self.croops_coord_top_left_groupbox)
        self.crops_coord_top_left_y.setObjectName(u"crops_coord_top_left_y")

        self.horizontalLayout_2.addWidget(self.crops_coord_top_left_y)


        self.gridLayout.addWidget(self.croops_coord_top_left_groupbox, 11, 0, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.label_2 = QLabel(self.scrollAreaWidgetContents)
        self.label_2.setObjectName(u"label_2")

        self.verticalLayout.addWidget(self.label_2)

        self.image_presets = QComboBox(self.scrollAreaWidgetContents)
        self.image_presets.setObjectName(u"image_presets")

        self.verticalLayout.addWidget(self.image_presets)

        self.line_2 = QFrame(self.scrollAreaWidgetContents)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setFrameShape(QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QFrame.Shadow.Sunken)

        self.verticalLayout.addWidget(self.line_2)


        self.horizontalLayout.addLayout(self.verticalLayout)

        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.label_3 = QLabel(self.scrollAreaWidgetContents)
        self.label_3.setObjectName(u"label_3")

        self.verticalLayout_2.addWidget(self.label_3)

        self.quality_effects = QComboBox(self.scrollAreaWidgetContents)
        self.quality_effects.addItem("")
        self.quality_effects.addItem("")
        self.quality_effects.addItem("")
        self.quality_effects.addItem("")
        self.quality_effects.setObjectName(u"quality_effects")

        self.verticalLayout_2.addWidget(self.quality_effects)

        self.line = QFrame(self.scrollAreaWidgetContents)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.verticalLayout_2.addWidget(self.line)


        self.horizontalLayout.addLayout(self.verticalLayout_2)


        self.gridLayout.addLayout(self.horizontalLayout, 2, 0, 1, 1)

        self.generator_form_splitter = QSplitter(self.scrollAreaWidgetContents)
        self.generator_form_splitter.setObjectName(u"generator_form_splitter")
        self.generator_form_splitter.setOrientation(Qt.Orientation.Vertical)
        self.generator_form_splitter.setChildrenCollapsible(False)
        self.layoutWidget = QWidget(self.generator_form_splitter)
        self.layoutWidget.setObjectName(u"layoutWidget")
        self.gridLayout_2 = QGridLayout(self.layoutWidget)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(0, 0, 0, 9)
        self.prompt = QPlainTextEdit(self.layoutWidget)
        self.prompt.setObjectName(u"prompt")

        self.gridLayout_2.addWidget(self.prompt, 1, 0, 1, 2)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.horizontalLayout_6.setContentsMargins(-1, 10, -1, -1)
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_6.addItem(self.horizontalSpacer)

        self.pushButton = QPushButton(self.layoutWidget)
        self.pushButton.setObjectName(u"pushButton")

        self.horizontalLayout_6.addWidget(self.pushButton)


        self.gridLayout_2.addLayout(self.horizontalLayout_6, 0, 1, 1, 1)

        self.secondary_prompt = QPlainTextEdit(self.layoutWidget)
        self.secondary_prompt.setObjectName(u"secondary_prompt")

        self.gridLayout_2.addWidget(self.secondary_prompt, 2, 0, 1, 2)

        self.label = QLabel(self.layoutWidget)
        self.label.setObjectName(u"label")
        font1 = QFont()
        font1.setPointSize(8)
        font1.setBold(True)
        self.label.setFont(font1)

        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)

        self.generator_form_splitter.addWidget(self.layoutWidget)
        self.layoutWidget1 = QWidget(self.generator_form_splitter)
        self.layoutWidget1.setObjectName(u"layoutWidget1")
        self.negative_prompt_container = QVBoxLayout(self.layoutWidget1)
        self.negative_prompt_container.setObjectName(u"negative_prompt_container")
        self.negative_prompt_container.setContentsMargins(0, 5, 0, 0)
        self.negative_prompt_label = QLabel(self.layoutWidget1)
        self.negative_prompt_label.setObjectName(u"negative_prompt_label")
        self.negative_prompt_label.setFont(font1)

        self.negative_prompt_container.addWidget(self.negative_prompt_label)

        self.negative_prompt = QPlainTextEdit(self.layoutWidget1)
        self.negative_prompt.setObjectName(u"negative_prompt")

        self.negative_prompt_container.addWidget(self.negative_prompt)

        self.secondary_negative_prompt = QPlainTextEdit(self.layoutWidget1)
        self.secondary_negative_prompt.setObjectName(u"secondary_negative_prompt")

        self.negative_prompt_container.addWidget(self.secondary_negative_prompt)

        self.generator_form_splitter.addWidget(self.layoutWidget1)

        self.gridLayout.addWidget(self.generator_form_splitter, 0, 0, 1, 1)

        self.stable_diffusion_generator_form.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout_3.addWidget(self.stable_diffusion_generator_form, 0, 0, 1, 1)

        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.horizontalLayout_7.setContentsMargins(10, 10, 10, 10)
        self.progress_bar = QProgressBar(self.tab)
        self.progress_bar.setObjectName(u"progress_bar")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.progress_bar.sizePolicy().hasHeightForWidth())
        self.progress_bar.setSizePolicy(sizePolicy)
        self.progress_bar.setMinimumSize(QSize(0, 30))
        self.progress_bar.setValue(0)

        self.horizontalLayout_7.addWidget(self.progress_bar)

        self.generate_button = QPushButton(self.tab)
        self.generate_button.setObjectName(u"generate_button")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.generate_button.sizePolicy().hasHeightForWidth())
        self.generate_button.setSizePolicy(sizePolicy1)
        self.generate_button.setMinimumSize(QSize(30, 30))
        self.generate_button.setMaximumSize(QSize(30, 30))
        icon = QIcon(QIcon.fromTheme(u"go-up"))
        self.generate_button.setIcon(icon)

        self.horizontalLayout_7.addWidget(self.generate_button)

        self.interrupt_button = QPushButton(self.tab)
        self.interrupt_button.setObjectName(u"interrupt_button")
        sizePolicy1.setHeightForWidth(self.interrupt_button.sizePolicy().hasHeightForWidth())
        self.interrupt_button.setSizePolicy(sizePolicy1)
        self.interrupt_button.setMinimumSize(QSize(30, 30))
        self.interrupt_button.setMaximumSize(QSize(30, 30))
        self.interrupt_button.setCursor(QCursor(Qt.PointingHandCursor))
        icon1 = QIcon(QIcon.fromTheme(u"application-exit"))
        self.interrupt_button.setIcon(icon1)

        self.horizontalLayout_7.addWidget(self.interrupt_button)


        self.gridLayout_3.addLayout(self.horizontalLayout_7, 1, 0, 1, 1)

        self.generator_form_tabs.addTab(self.tab, "")
        self.tab_2 = QWidget()
        self.tab_2.setObjectName(u"tab_2")
        self.gridLayout_5 = QGridLayout(self.tab_2)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.gridLayout_7 = QGridLayout()
        self.gridLayout_7.setObjectName(u"gridLayout_7")
        self.gridLayout_7.setHorizontalSpacing(0)
        self.gridLayout_7.setVerticalSpacing(10)
        self.chat_prompt_widget = ChatPromptWidget(self.tab_2)
        self.chat_prompt_widget.setObjectName(u"chat_prompt_widget")

        self.gridLayout_7.addWidget(self.chat_prompt_widget, 0, 0, 1, 1)


        self.gridLayout_5.addLayout(self.gridLayout_7, 0, 0, 1, 1)

        self.generator_form_tabs.addTab(self.tab_2, "")
        self.llm_history_widget_tab = QWidget()
        self.llm_history_widget_tab.setObjectName(u"llm_history_widget_tab")
        self.gridLayout_6 = QGridLayout(self.llm_history_widget_tab)
        self.gridLayout_6.setSpacing(0)
        self.gridLayout_6.setObjectName(u"gridLayout_6")
        self.gridLayout_6.setContentsMargins(0, 0, 0, 0)
        self.llm_history_widget = LLMHistoryWidget(self.llm_history_widget_tab)
        self.llm_history_widget.setObjectName(u"llm_history_widget")

        self.gridLayout_6.addWidget(self.llm_history_widget, 0, 0, 1, 1)

        self.generator_form_tabs.addTab(self.llm_history_widget_tab, "")
        self.chatbot_mood_tab = QWidget()
        self.chatbot_mood_tab.setObjectName(u"chatbot_mood_tab")
        self.gridLayout_8 = QGridLayout(self.chatbot_mood_tab)
        self.gridLayout_8.setSpacing(0)
        self.gridLayout_8.setObjectName(u"gridLayout_8")
        self.gridLayout_8.setContentsMargins(0, 0, 0, 0)
        self.scrollArea = QScrollArea(self.chatbot_mood_tab)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents_2 = QWidget()
        self.scrollAreaWidgetContents_2.setObjectName(u"scrollAreaWidgetContents_2")
        self.scrollAreaWidgetContents_2.setGeometry(QRect(0, 0, 614, 918))
        self.gridLayout_9 = QGridLayout(self.scrollAreaWidgetContents_2)
        self.gridLayout_9.setObjectName(u"gridLayout_9")
        self.mood_label = QLabel(self.scrollAreaWidgetContents_2)
        self.mood_label.setObjectName(u"mood_label")
        self.mood_label.setWordWrap(True)

        self.gridLayout_9.addWidget(self.mood_label, 0, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_9.addItem(self.verticalSpacer, 1, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents_2)

        self.gridLayout_8.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.generator_form_tabs.addTab(self.chatbot_mood_tab, "")

        self.gridLayout_4.addWidget(self.generator_form_tabs, 0, 0, 1, 1)

        QWidget.setTabOrder(self.generator_form_tabs, self.stable_diffusion_generator_form)
        QWidget.setTabOrder(self.stable_diffusion_generator_form, self.prompt)
        QWidget.setTabOrder(self.prompt, self.secondary_prompt)
        QWidget.setTabOrder(self.secondary_prompt, self.negative_prompt)
        QWidget.setTabOrder(self.negative_prompt, self.secondary_negative_prompt)
        QWidget.setTabOrder(self.secondary_negative_prompt, self.pushButton)
        QWidget.setTabOrder(self.pushButton, self.crops_coord_top_left_x)
        QWidget.setTabOrder(self.crops_coord_top_left_x, self.crops_coord_top_left_y)
        QWidget.setTabOrder(self.crops_coord_top_left_y, self.generate_button)
        QWidget.setTabOrder(self.generate_button, self.interrupt_button)

        self.retranslateUi(generator_form)
        self.pushButton.clicked.connect(generator_form.action_clicked_button_save_prompts)
        self.interrupt_button.clicked.connect(generator_form.handle_interrupt_button_clicked)
        self.generate_button.clicked.connect(generator_form.handle_generate_button_clicked)
        self.prompt.textChanged.connect(generator_form.handle_prompt_changed)
        self.negative_prompt.textChanged.connect(generator_form.handle_negative_prompt_changed)
        self.image_presets.currentTextChanged.connect(generator_form.handle_image_presets_changed)
        self.secondary_prompt.textChanged.connect(generator_form.handle_second_prompt_changed)
        self.secondary_negative_prompt.textChanged.connect(generator_form.handle_second_prompt_changed)
        self.quality_effects.textActivated.connect(generator_form.handle_quality_effects_changed)

        self.generator_form_tabs.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(generator_form)
    # setupUi

    def retranslateUi(self, generator_form):
        generator_form.setWindowTitle(QCoreApplication.translate("generator_form", u"Form", None))
        self.croops_coord_top_left_groupbox.setTitle(QCoreApplication.translate("generator_form", u"Crops Coord (top left)", None))
        self.crops_coord_top_left_x.setPlaceholderText(QCoreApplication.translate("generator_form", u"x position", None))
        self.crops_coord_top_left_y.setPlaceholderText(QCoreApplication.translate("generator_form", u"y position", None))
        self.label_2.setText(QCoreApplication.translate("generator_form", u"Presets", None))
        self.label_3.setText(QCoreApplication.translate("generator_form", u"Quality Effects", None))
        self.quality_effects.setItemText(0, "")
        self.quality_effects.setItemText(1, QCoreApplication.translate("generator_form", u"Normal", None))
        self.quality_effects.setItemText(2, QCoreApplication.translate("generator_form", u"Upscaled", None))
        self.quality_effects.setItemText(3, QCoreApplication.translate("generator_form", u"Downscaled", None))

        self.prompt.setPlaceholderText(QCoreApplication.translate("generator_form", u"Enter a prompt...", None))
        self.pushButton.setText(QCoreApplication.translate("generator_form", u"Save Prompts", None))
        self.secondary_prompt.setPlaceholderText(QCoreApplication.translate("generator_form", u"Enter a second prompt...", None))
        self.label.setText(QCoreApplication.translate("generator_form", u"Prompt", None))
        self.negative_prompt_label.setText(QCoreApplication.translate("generator_form", u"Negative Prompt", None))
        self.negative_prompt.setPlaceholderText(QCoreApplication.translate("generator_form", u"Enter a negative prompt...", None))
        self.secondary_negative_prompt.setPlaceholderText(QCoreApplication.translate("generator_form", u"Enter a second negative prompt...", None))
#if QT_CONFIG(tooltip)
        self.generate_button.setToolTip(QCoreApplication.translate("generator_form", u"Generate image", None))
#endif // QT_CONFIG(tooltip)
        self.generate_button.setText("")
#if QT_CONFIG(tooltip)
        self.interrupt_button.setToolTip(QCoreApplication.translate("generator_form", u"Cancel image generation", None))
#endif // QT_CONFIG(tooltip)
        self.interrupt_button.setText("")
        self.generator_form_tabs.setTabText(self.generator_form_tabs.indexOf(self.tab), QCoreApplication.translate("generator_form", u"AI Art", None))
        self.generator_form_tabs.setTabText(self.generator_form_tabs.indexOf(self.tab_2), QCoreApplication.translate("generator_form", u"Chat", None))
        self.generator_form_tabs.setTabText(self.generator_form_tabs.indexOf(self.llm_history_widget_tab), QCoreApplication.translate("generator_form", u"Chat History", None))
        self.mood_label.setText(QCoreApplication.translate("generator_form", u"TextLabel", None))
        self.generator_form_tabs.setTabText(self.generator_form_tabs.indexOf(self.chatbot_mood_tab), QCoreApplication.translate("generator_form", u"Chatbot mood", None))
    # retranslateUi




#!/usr/bin/env python

import facehuggershield
facehuggershield.huggingface.activate(
    show_stdout=True,
    darklock_os_whitelisted_operations=[
        "makedirs"
    ]
)
from airunner.app_installer import AppInstaller


if __name__ == "__main__":
    AppInstaller()



import os
from queue import Queue
import time
import requests
from PySide6.QtCore import QObject, Signal
from airunner.aihandler.logger import Logger
from airunner.enums import SignalCode
from airunner.mediator_mixin import MediatorMixin
from facehuggershield.huggingface.settings import DEFAULT_HF_ENDPOINT
from airunner.windows.main.settings_mixin import SettingsMixin

logger = Logger(prefix="DownloadWorker")


class DownloadWorker(
    QObject,
    MediatorMixin,
    SettingsMixin
):
    progress = Signal(int, int)  # current, total
    finished = Signal()
    failed = Signal(Exception)
    queue = Queue()
    running = False
    is_cancelled = False

    def __init__(self, *args, **kwargs):
        MediatorMixin.__init__(self)
        SettingsMixin.__init__(self)
        super(DownloadWorker, self).__init__(*args, **kwargs)

    def add_to_queue(self, data: tuple):
        self.queue.put(data)

    def get_size(self, url: str):
        try:
            response = requests.head(url, allow_redirects=True)
            size_kb = int(response.headers.get("content-length", 0))
            return size_kb
        except OverflowError:
            print(f"OverflowError when getting size for {url}")
            raise

    def cancel(self):
        self.is_cancelled = True

    def download(self):
        self.running = True

        while self.running:
            if self.queue.empty():
                time.sleep(0.1)
                continue

            queue_size = self.queue.qsize()

            path, file_name, file_path, callback = self.queue.get()
            if path == "" and file_name == "" and file_path == "":
                callback()
                return
            url = f"{DEFAULT_HF_ENDPOINT}/{path}/resolve/main/{file_name}?download=true".replace(" ", "")
            self.emit_signal(SignalCode.CLEAR_DOWNLOAD_STATUS_BAR)
            self.emit_signal(SignalCode.SET_DOWNLOAD_STATUS_LABEL, {
                "message": f"Downloading {file_name}"
            })

            file_name = os.path.join(file_path, file_name)
            file_name = os.path.expanduser(file_name)

            if not os.path.exists(os.path.dirname(file_name)):
                os.makedirs(os.path.dirname(file_name), exist_ok=True)

            if os.path.exists(file_name):
                self.emit_signal(
                    SignalCode.UPDATE_DOWNLOAD_LOG,
                    {
                        "message":  f"File already exists, skipping download"
                    }
                )
                self.emit_signal(SignalCode.DOWNLOAD_PROGRESS, {
                    "current": 0,
                    "total": 0
                })
                self.finished.emit()
                continue

            size_kb = self.get_size(url)
            self.emit_signal(
                SignalCode.UPDATE_DOWNLOAD_LOG,
                {
                    "message": f"Downloading {url} of size {size_kb} KB to {file_name}"
                }
            )

            try:
                headers = {}
                with requests.get(url, headers=headers, stream=True, allow_redirects=True) as r:
                    r.raise_for_status()
                    dir_name = os.path.dirname(file_name)

                    if dir_name != "" and not os.path.exists(dir_name):
                        os.makedirs(dir_name, exist_ok=True)

                    with open(file_name, "wb") as f:
                        for chunk in r.iter_content(chunk_size=8192):
                            if self.is_cancelled:
                                break
                            try:
                                f.write(chunk)
                            except OverflowError:
                                print(f"OverflowError when writing to {file_name}")
                                raise
                            self.emit_signal(SignalCode.DOWNLOAD_PROGRESS, {
                                "current": f.tell(),
                                "total": size_kb
                            })
                        self.emit_signal(
                            SignalCode.UPDATE_DOWNLOAD_LOG,
                            {
                                "message": f"finished with download of {file_name}"
                            }
                        )
                        self.finished.emit()
            except Exception as e:
                print(f"Failed to download {url}")
                print(e)
                self.failed.emit(e)
                self.emit_signal(SignalCode.DOWNLOAD_COMPLETE)



from airunner.app import App


class SocketServer(App):
    def __init__(
        self,
        *args,
        # host="",
        # port="",
        # keep_alive="",
        # packet_size=""
    ):
        super().__init__(*args)

    def run(self):
        pass



"""
----------------------------------------------------------------
Import order is crucial for AI Runner to work as expected.
Do not remove the no_internet_socket import.
Do not change the order of the imports.
----------------------------------------------------------------
"""
################################################################
# Import the main application class for AI Runner.
################################################################
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QApplication

from airunner.windows.download_wizard.download_wizard_window import DownloadWizardWindow


if __name__ == "__main__":
    # Run the AI Runner application.
    QApplication.setAttribute(Qt.ApplicationAttribute.AA_UseDesktopOpenGL)
    app = QApplication([])
    download_wizard = DownloadWizardWindow()
    download_wizard.exec()



import os

from PySide6.QtCore import Slot, QSize, QThread
from PySide6.QtWidgets import QWidget, QSizePolicy

from airunner.enums import SignalCode, ModelType, ModelStatus
from airunner.utils.models.scan_path_for_items import scan_path_for_lora
from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.lora.lora_widget import LoraWidget
from airunner.widgets.lora.templates.lora_container_ui import Ui_lora_container
from airunner.workers.directory_watcher import DirectoryWatcher


class LoraContainerWidget(BaseWidget):
    widget_class_ = Ui_lora_container
    search_filter = ""
    spacer = None

    def __init__(self, *args, **kwargs):
        self._version = None
        super().__init__(*args, **kwargs)
        self.initialized = False
        self._deleting = False
        self.register(SignalCode.APPLICATION_SETTINGS_CHANGED_SIGNAL, self.on_application_settings_changed_signal)
        self.register(SignalCode.LORA_UPDATED_SIGNAL, self.on_lora_updated_signal)
        self.register(SignalCode.MODEL_STATUS_CHANGED_SIGNAL, self.on_model_status_changed_signal)
        self.register(SignalCode.LORA_STATUS_CHANGED, self.on_lora_modified)
        self.register(SignalCode.LORA_DELETE_SIGNAL, self.delete_lora)
        self.ui.loading_icon.hide()
        self.ui.loading_icon.set_size(spinner_size=QSize(30, 30), label_size=QSize(24, 24))
        self._apply_button_enabled = False
        self.ui.apply_lora_button.setEnabled(self._apply_button_enabled)
        self._scanner_worker = DirectoryWatcher(self.path_settings.base_path, self._scan_path_for_lora)
        self._scanner_worker.scan_completed.connect(self.on_scan_completed)
        self._scanner_thread = QThread()
        self._scanner_worker.moveToThread(self._scanner_thread)
        self._scanner_thread.started.connect(self._scanner_worker.run)
        self._scanner_thread.start()

    def _scan_path_for_lora(self, path) -> bool:
        if self._deleting:
            return False
        return scan_path_for_lora(path)

    @Slot(bool)
    def on_scan_completed(self, force_reload: bool):
        self.load_lora(force_reload=force_reload)

    @Slot()
    def scan_for_lora(self):
        # clear all lora widgets
        force_reload = scan_path_for_lora(self.path_settings.base_path)
        self.load_lora(force_reload=force_reload)

    @Slot()
    def apply_lora(self):
        self._apply_button_enabled = False
        self.emit_signal(SignalCode.LORA_UPDATE_SIGNAL)

    def on_lora_modified(self):
        self._apply_button_enabled = True
        self.ui.apply_lora_button.setEnabled(self._apply_button_enabled)

    def on_model_status_changed_signal(self, data):
        model = data["model"]
        status = data["status"]
        if model is ModelType.SD:
            if status is ModelStatus.LOADING:
                self._disable_form()
            else:
                self._enable_form()

    def _disable_form(self):
        self.ui.apply_lora_button.setEnabled(self._apply_button_enabled)
        self.ui.lora_scale_slider.setEnabled(False)
        self.ui.toggleAllLora.setEnabled(False)
        self.ui.loading_icon.show()
        self._toggle_lora_widgets(False)

    def _enable_form(self):
        self.ui.apply_lora_button.setEnabled(self._apply_button_enabled)
        self.ui.lora_scale_slider.setEnabled(True)
        self.ui.toggleAllLora.setEnabled(True)
        self.ui.loading_icon.hide()
        self._toggle_lora_widgets(True)

    def _toggle_lora_widgets(self, enable: bool):
        for i in range(self.ui.lora_scroll_area.widget().layout().count()):
            lora_widget = self.ui.lora_scroll_area.widget().layout().itemAt(i).widget()
            if isinstance(lora_widget, LoraWidget):
                if enable:
                    lora_widget.enable_lora_widget()
                else:
                    lora_widget.disable_lora_widget()

    def on_application_settings_changed_signal(self):
        self.load_lora()

    def on_lora_updated_signal(self):
        self._enable_form()

    def toggle_all(self, val):
        lora_widgets = [
            self.ui.lora_scroll_area.widget().layout().itemAt(i).widget()
            for i in range(self.ui.lora_scroll_area.widget().layout().count())
            if isinstance(self.ui.lora_scroll_area.widget().layout().itemAt(i).widget(), LoraWidget)
        ]
        for lora_widget in lora_widgets:
            lora_widget.ui.enabledCheckbox.blockSignals(True)
            lora_widget.action_toggled_lora_enabled(val)
            lora_widget.ui.enabledCheckbox.blockSignals(False)

    def showEvent(self, event):
        if not self.initialized:
            self.scan_for_lora()
            self.initialized = True
        self.load_lora()

    def load_lora(self, force_reload=False):
        version = self.generator_settings.version

        if self._version is None or self._version != version or force_reload:
            self._version = version
            self.clear_lora_widgets()
            loras = self.get_lora_by_version(self._version)
            if loras:
                self.remove_spacer()
                filtered_loras = [
                    lora for lora in loras
                    if self.search_filter.lower() in lora.name.lower()
                ]
                for lora in filtered_loras:
                    self.add_lora(lora)
                self.add_spacer()

    def remove_spacer(self):
        # remove spacer from end of self.ui.scrollAreaWidgetContents.layout()
        if self.spacer:
            self.ui.scrollAreaWidgetContents.layout().removeWidget(self.spacer)

    def add_spacer(self):
        # add spacer to end of self.ui.scrollAreaWidgetContents.layout()
        if not self.spacer:
            self.spacer = QWidget()
            self.spacer.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        else:
            self.remove_spacer()
        self.ui.scrollAreaWidgetContents.layout().addWidget(self.spacer)

    def add_lora(self, lora):
        if lora is None:
            return
        lora_widget = LoraWidget(lora=lora)
        self.ui.scrollAreaWidgetContents.layout().addWidget(lora_widget)

    def delete_lora(self, data: dict):
        self._deleting = True
        lora_widget = data["lora_widget"]

        # Delete the lora from disc
        lora_path = os.path.expanduser(
            os.path.join(
                self.path_settings.base_path,
                "art/models",
                self._version,
                "lora"
            )
        )
        lora_file = lora_widget.current_lora.name
        for dirpath, dirnames, filenames in os.walk(lora_path):
            for file in filenames:
                if file.startswith(lora_file):
                    os.remove(os.path.join(dirpath, file))
                    break

        # Remove lora from database
        session = self.db_handler.get_db_session()
        session.delete(lora_widget.current_lora)
        session.commit()
        session.close()

        self._apply_button_enabled = True
        self.ui.apply_lora_button.setEnabled(self._apply_button_enabled)
        self.load_lora(force_reload=True)
        self._deleting = False

    def available_lora(self, action):
        available_lora = []
        for lora in self.lora:
            if lora.enabled and lora.scale > 0:
                available_lora.append(lora)
        return available_lora

    def initialize_lora_trigger_words(self):
        for lora in self.lora:
            trigger_word = lora["trigger_word"] if "trigger_word" in lora else ""
            for tab_name in self.tabs.keys():
                tab = self.tabs[tab_name]
                for i in range(self.tool_menu_widget.lora_container_widget.lora_scroll_area.widget().layout().count()):
                    lora_widget = self.tool_menu_widget.lora_container_widget.lora_scroll_area.widget().layout().itemAt(
                        i).widget()
                    if not lora_widget:
                        continue
                    if lora_widget.enabledCheckbox.text() == lora["name"]:
                        if trigger_word != "":
                            lora_widget.trigger_word.setText(trigger_word)
                        lora_widget.trigger_word.textChanged.connect(
                            lambda value, _lora_widget=lora_widget, _lora=lora,
                                   _tab_name=tab_name: self.handle_lora_trigger_word(_lora, _lora_widget, value))
                        break

    def handle_lora_trigger_word(self, lora, lora_widget, value):
        for n in range(len(self.lora)):
            lora_object = self.lora[n]
            if lora_object.name == lora.name:
                lora_object.trigger_word = value
                self.update_lora(lora_object)

    def handle_lora_slider(self, lora, lora_widget, value, tab_name):
        float_val = value / 100
        for n in range(len(self.lora)):
            lora_object = self.lora[n]
            if lora_object.name == lora.name:
                lora_object.scale = float_val
                self.update_lora(lora_object)
        lora_widget.scaleSpinBox.setValue(float_val)

    def handle_lora_spinbox(self, lora, lora_widget, value, tab_name):
        for n in range(len(self.lora)):
            lora_object = self.lora[n]
            if lora_object.name == lora.name:
                lora_object.scale = value
                self.update_lora(lora_object)
        lora_widget.scaleSlider.setValue(int(value * 100))

    def search_text_changed(self, val):
        self.search_filter = val
        self.load_lora(force_reload=True)
    
    def clear_lora_widgets(self):
        if self.spacer:
            self.ui.scrollAreaWidgetContents.layout().removeWidget(self.spacer)
        for i in reversed(range(self.ui.scrollAreaWidgetContents.layout().count())):
            widget = self.ui.scrollAreaWidgetContents.layout().itemAt(i).widget()
            if isinstance(widget, LoraWidget):
                widget.deleteLater()



import pyttsx3

from airunner.settings import ESPEAK_SETTINGS
from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.tts.templates.espeak_preferences_ui import Ui_espeak_preferences
import pycountry

class ESpeakPreferencesWidget(BaseWidget):
    widget_class_ = Ui_espeak_preferences

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def initialize_form(self):
        elements = [
            self.ui.language_combobox,
            self.ui.gender_combobox,
            self.ui.voice_combobox,
        ]

        for element in elements:
            element.blockSignals(True)

        language = self.espeak_settings.language
        gender = self.espeak_settings.gender
        voice = self.espeak_settings.voice
        iso_codes = [country.alpha_2 for country in pycountry.countries]

        engine = pyttsx3.init()
        voices = engine.getProperty("voices")
        voice_names = [voice.name for voice in voices]

        self.ui.language_combobox.clear()
        self.ui.language_combobox.addItems(iso_codes)
        self.ui.language_combobox.setCurrentText(language)
        self.ui.gender_combobox.clear()
        self.ui.gender_combobox.addItems(["Male", "Female"])
        self.ui.gender_combobox.setCurrentText(gender)
        self.ui.voice_combobox.clear()
        self.ui.voice_combobox.addItems(voice_names)
        self.ui.voice_combobox.setCurrentText(voice)

        for element in elements:
            element.blockSignals(False)

        self.ui.rate.init(slider_callback=self.callback, current_value=self.espeak_settings.rate)
        self.ui.volume.init(slider_callback=self.callback, current_value=self.espeak_settings.volume)
        self.ui.pitch.init(slider_callback=self.callback, current_value=self.espeak_settings.pitch)

    def callback(self, attr_name, value, widget=None):
        self.update_espeak_settings(attr_name, value)

    def language_changed(self, text):
        self.update_espeak_settings("language", text)
        self.update_espeak_settings("gender", self.ui.gender_combobox.currentText())
        self.update_espeak_settings("voice", self.ui.voice_combobox.currentText())

        self.update_espeak_settings("language", text)
        self.update_espeak_settings("gender", self.ui.gender_combobox.currentText())
        self.update_espeak_settings("voice", self.ui.voice_combobox.currentText())

    def voice_changed(self, text):
        self.update_espeak_settings("voice", text)

    def gender_changed(self, text):
        self.update_espeak_settings("gender", text)
        self.ui.voice_combobox.clear()
        self.ui.voice_combobox.addItems(ESPEAK_SETTINGS["voices"][text.lower()])
        self.update_espeak_settings("voice", self.ui.voice_combobox.currentText())



import os
import re
import subprocess
import time
from pathlib import Path

from PySide6.QtCore import QObject, Signal


def build_ui(path):
    """Build the UI for the application."""
    # venv_python = get_venv_python_executable()
    # recursively iterate over directories in path
    print("Building UI at path", path)
    ui_files = Path(__file__).parent.parent.joinpath(path).glob("**/*.ui")
    for ui_file in ui_files:

        print("Generating", ui_file)
        ui_file = str(ui_file)
        ui_file_name = os.path.basename(ui_file)
        ui_file_dir = os.path.dirname(ui_file)
        ui_file_py = ui_file.replace(".ui", "_ui.py")
        print(f"Generating {ui_file_py}")
        subprocess.run(
            [
                "pyuic6",
                "-o",
                ui_file_py,
                ui_file,
            ],
            cwd=ui_file_dir,
            )


def generate_resources():
    print("Generating resources.py")
    here = os.path.dirname(os.path.abspath(__file__))
    subprocess.run(
        [
            "pyside6-rcc",
            "-o",
            os.path.join(here, "resources_rc.py"),
            os.path.join(here, "resources_dark.qrc"),
        ],
        cwd=str(Path(__file__).parent.parent),
    )


def build_templates(path):
    build_ui(path)
    generate_resources()


def process_qss(_path=None):
    # Define the regular expression pattern for variables
    VAR_BLOCK_PATTERN = r'/\*\s*VARIABLES\s*\*/(.+?)/\*\s*END_VARIABLES\s*\*/'
    VAR_PATTERN = r'@[\w-]+'

    def process_file(file_path, output_file, variables):
        """Process a single QSS file and write the output to the output file."""
        with open(file_path, 'r') as f:
            contents = f.read()

        # Replace any include statements with the contents of the included file
        contents = re.sub(r'\$include\("(.+)"\)', lambda m: process_file(m.group(1), output_file, variables), contents)

        # Replace any variables with their values
        for match in re.finditer(VAR_PATTERN, contents):
            var_name = match.group(0)
            if var_name in variables:
                contents = contents.replace(var_name, variables[var_name])

        # Write the processed contents to the output file
        output_file.write(contents)

    def process_manifest(manifest_path, output_dir):
        """Process a single manifest file and write the output to the output file."""
        with open(manifest_path, 'r') as f:
            contents = f.read()

        # Process each QSS file listed in the manifest
        output_path = os.path.join(output_dir, 'styles.qss')
        print(output_path)
        variables = {}
        with open(output_path, 'w') as output_file:
            for filename in contents.splitlines():
                file_path = os.path.join(os.path.dirname(manifest_path), filename.strip())
                if os.path.isfile(file_path) and filename.endswith('.qss'):
                    with open(file_path, 'r') as input_file:
                        input_contents = input_file.read()
                    var_block_match = re.search(VAR_BLOCK_PATTERN, input_contents, flags=re.DOTALL)
                    if var_block_match:
                        var_block_contents = var_block_match.group(1)
                        for match in re.finditer(VAR_PATTERN, var_block_contents):
                            var_name = match.group(0)
                            if var_name not in variables:
                                variables[var_name] = get_variable_value(var_name, var_block_contents)
                    process_file(file_path, output_file, variables)
        # Remove variable blocks from the output file
        with open(output_path, 'r') as f:
            contents = f.read()
        contents = re.sub(VAR_BLOCK_PATTERN, '', contents, flags=re.DOTALL)
        with open(output_path, 'w') as f:
            f.write(contents)

    def get_variable_value(var_name, contents):
        """Get the value of a variable by searching for its definition in the contents."""
        var_pattern = re.escape(var_name) + r'\s*:\s*([^;]+);'
        match = re.search(var_pattern, contents)
        if match:
            return match.group(1).strip()
        else:
            return var_name

    def process_directory(directory_path):
        """Recursively process all manifest files in a directory and its subdirectories."""
        for filename in os.listdir(directory_path):
            file_path = os.path.join(directory_path, filename)
            if os.path.isdir(file_path):
                output_dir = os.path.join(directory_path, filename)
                process_directory(file_path)
            elif os.path.isfile(file_path) and filename == 'manifest.txt':
                output_dir = os.path.dirname(file_path)
                process_manifest(file_path, output_dir)

    # Process all manifest files in the styles directory and its subdirectories
    process_directory(os.path.join(os.getcwd(), 'styles'))


class SignalEmitter(QObject):
    file_changed = Signal()



import threading

from PySide6 import QtCore
from PySide6.QtCore import QTimer

from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.model_manager.model_widget import ModelWidget
from airunner.widgets.model_manager.templates.default_ui import Ui_default_model_widget
from PySide6 import QtWidgets
from airunner.windows.main.ai_model_mixin import AIModelMixin
from airunner.windows.main.pipeline_mixin import PipelineMixin


class DefaultModelWidget(
    BaseWidget,
    PipelineMixin,
    AIModelMixin
):
    widget_class_ = Ui_default_model_widget
    model_widgets = []

    def __init__(self, *args, **kwargs):
        self.initialized = False
        PipelineMixin.__init__(self)
        AIModelMixin.__init__(self)
        super().__init__(*args, **kwargs)
        self.spacer = None
        self.initialized = False
        self.__thread = threading.Thread(target=self.__do_show)

    def showEvent(self, event):
        super().showEvent(event)
        QTimer.singleShot(
            50,
            self.__do_show
        )

    def __do_show(self):
        if not self.initialized:
            self.show_items_in_scrollarea()
            # find how many models are set to enabled = False
            self.ui.toggle_all.blockSignals(True)
            disabled_models = self.ai_model_get_disabled_default()
            if len(disabled_models) == 0:
                self.ui.toggle_all.setCheckState(QtCore.Qt.CheckState.Checked)
            elif len(disabled_models) < len(self.ai_models_find(default=True)):
                self.ui.toggle_all.setCheckState(QtCore.Qt.CheckState.PartiallyChecked)
            else:
                self.ui.toggle_all.setCheckState(QtCore.Qt.CheckState.Unchecked)
            self.ui.toggle_all.blockSignals(False)
            self.initialized = True

    def show_items_in_scrollarea(self, search=None):
        if self.spacer:
            self.ui.scrollAreaWidgetContents.layout().removeItem(self.spacer)
        for child in self.ui.scrollAreaWidgetContents.children():
            if isinstance(child, ModelWidget):
                child.deleteLater()
        if search:
            # search by name
            models = self.ai_models_find(search, default=True)
        else:
            models = self.ai_models_find(default=True)
        for model_widget in self.model_widgets:
            model_widget.deleteLater()
        self.model_widgets = []
        for index, model in enumerate(models):
            version = model["version"]
            category = model["category"]
            pipeline_action = model["pipeline_action"]
            pipeline_class = self.get_pipeline_classname(
                pipeline_action,
                version,
                category
            )
            model_widget = ModelWidget(
                path=model["path"],
                branch=model["branch"],
                version=version,
                category=category,
                pipeline_action=pipeline_action,
                pipeline_class=pipeline_class,
            )
            model_widget.ui.delete_button.hide()
            model_widget.ui.name.setChecked(model["enabled"])
            self.ui.scrollAreaWidgetContents.layout().addWidget(model_widget)
            self.model_widgets.append(model_widget)
        if not self.spacer:
            width = 20
            height = 40
            self.spacer = QtWidgets.QSpacerItem(
                width,
                height,
                QtWidgets.QSizePolicy.Policy.Minimum,
                QtWidgets.QSizePolicy.Policy.Expanding
            )
        self.ui.scrollAreaWidgetContents.layout().addItem(self.spacer)

    def mode_type_changed(self, val):
        print("mode_type_changed", val)
    
    def toggle_all_state_change(self, val: int):
        if val == 0:
            # disable all by setting AIModel.enabled to False where is_default=True
            for item in self.ai_models:
                item.enabled = False
                self.update_ai_model(item)
            self.show_items_in_scrollarea()
        elif val == 1:
            # self.ui.toggle_all is a checkbox with tri-state enabled, how can we set it to checked?
            self.ui.toggle_all.setCheckState(QtCore.Qt.CheckState.Checked)
        elif val == 2:
            for item in self.ai_models:
                item.enabled = True
                self.update_ai_model(item)
            self.show_items_in_scrollarea()

    def search_text_changed(self, val):
        val = val.strip()
        if val == "":
            val = None
        self.show_items_in_scrollarea(val)



from PySide6.QtCore import QObject, Signal

from airunner.mediator_mixin import MediatorMixin
from airunner.windows.main.settings_mixin import SettingsMixin
from airunner.aihandler.logger import Logger
from airunner.utils.create_worker import create_worker
from airunner.workers.audio_capture_worker import AudioCaptureWorker
from airunner.workers.audio_processor_worker import AudioProcessorWorker
from airunner.workers.llm_generate_worker import LLMGenerateWorker
from airunner.workers.mask_generator_worker import MaskGeneratorWorker
from airunner.workers.sd_worker import SDWorker
from airunner.workers.tts_generator_worker import TTSGeneratorWorker
from airunner.workers.tts_vocalizer_worker import TTSVocalizerWorker


class WorkerManager(QObject, MediatorMixin, SettingsMixin):
    """
    The engine is responsible for processing requests and offloading
    them to the appropriate AI model controller.
    """
    # Signals
    request_signal_status = Signal(str)
    image_generated_signal = Signal(dict)

    def __init__(
        self,
        disable_sd: bool = False,
        disable_llm: bool = False,
        disable_tts: bool = False,
        disable_stt: bool = False,
        agent_options: dict = None
    ):
        MediatorMixin.__init__(self)
        SettingsMixin.__init__(self)
        super().__init__()
        self.logger = Logger(prefix=self.__class__.__name__)
        self._sd_worker = None
        self._llm_request_worker = None
        self._llm_generate_worker = None
        self._tts_generator_worker = None
        self._tts_vocalizer_worker = None
        self._stt_audio_capture_worker = None
        self._stt_audio_processor_worker = None

        self.agent_options = agent_options

        if not disable_sd:
            self.register_sd_workers()

        if not disable_llm:
            self.register_llm_workers(self.agent_options)

        if not disable_tts:
            self.register_tts_workers()

        if not disable_stt:
            self.register_stt_workers()

        self.mask_generator_worker = create_worker(MaskGeneratorWorker)

    def register_sd_workers(self):
        self._sd_worker = create_worker(SDWorker)

    def register_llm_workers(self, agent_options):
        self._llm_generate_worker = create_worker(LLMGenerateWorker, agent_options=agent_options)

    def register_tts_workers(self):
        self._tts_generator_worker = create_worker(TTSGeneratorWorker)
        self._tts_vocalizer_worker = create_worker(TTSVocalizerWorker)

    def register_stt_workers(self):
        self._stt_audio_capture_worker = create_worker(AudioCaptureWorker)
        self._stt_audio_processor_worker = create_worker(AudioProcessorWorker)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'model_form.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QFrame,
    QGridLayout, QHBoxLayout, QLabel, QLineEdit,
    QSizePolicy, QSpacerItem, QWidget)

class Ui_model_form_widget(object):
    def setupUi(self, model_form_widget):
        if not model_form_widget.objectName():
            model_form_widget.setObjectName(u"model_form_widget")
        model_form_widget.resize(536, 688)
        self.gridLayout_5 = QGridLayout(model_form_widget)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.model_data = QGridLayout()
        self.model_data.setObjectName(u"model_data")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.label_3 = QLabel(model_form_widget)
        self.label_3.setObjectName(u"label_3")
        font = QFont()
        font.setBold(True)
        self.label_3.setFont(font)

        self.horizontalLayout.addWidget(self.label_3)

        self.poi = QLabel(model_form_widget)
        self.poi.setObjectName(u"poi")

        self.horizontalLayout.addWidget(self.poi)


        self.model_data.addLayout(self.horizontalLayout, 0, 0, 1, 1)

        self.line_2 = QFrame(model_form_widget)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setFrameShape(QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QFrame.Shadow.Sunken)

        self.model_data.addWidget(self.line_2, 1, 0, 1, 1)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.label_9 = QLabel(model_form_widget)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setFont(font)

        self.horizontalLayout_2.addWidget(self.label_9)

        self.require_credit = QLabel(model_form_widget)
        self.require_credit.setObjectName(u"require_credit")

        self.horizontalLayout_2.addWidget(self.require_credit)


        self.model_data.addLayout(self.horizontalLayout_2, 2, 0, 1, 1)

        self.line_3 = QFrame(model_form_widget)
        self.line_3.setObjectName(u"line_3")
        self.line_3.setFrameShape(QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QFrame.Shadow.Sunken)

        self.model_data.addWidget(self.line_3, 3, 0, 1, 1)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.label_11 = QLabel(model_form_widget)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setFont(font)

        self.horizontalLayout_3.addWidget(self.label_11)

        self.allow_commercial_use = QLabel(model_form_widget)
        self.allow_commercial_use.setObjectName(u"allow_commercial_use")

        self.horizontalLayout_3.addWidget(self.allow_commercial_use)


        self.model_data.addLayout(self.horizontalLayout_3, 4, 0, 1, 1)

        self.line_4 = QFrame(model_form_widget)
        self.line_4.setObjectName(u"line_4")
        self.line_4.setFrameShape(QFrame.Shape.HLine)
        self.line_4.setFrameShadow(QFrame.Shadow.Sunken)

        self.model_data.addWidget(self.line_4, 5, 0, 1, 1)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.label_14 = QLabel(model_form_widget)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setFont(font)

        self.horizontalLayout_4.addWidget(self.label_14)

        self.allow_derivatives = QLabel(model_form_widget)
        self.allow_derivatives.setObjectName(u"allow_derivatives")

        self.horizontalLayout_4.addWidget(self.allow_derivatives)


        self.model_data.addLayout(self.horizontalLayout_4, 6, 0, 1, 1)

        self.line_5 = QFrame(model_form_widget)
        self.line_5.setObjectName(u"line_5")
        self.line_5.setFrameShape(QFrame.Shape.HLine)
        self.line_5.setFrameShadow(QFrame.Shadow.Sunken)

        self.model_data.addWidget(self.line_5, 7, 0, 1, 1)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.label_16 = QLabel(model_form_widget)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setFont(font)

        self.horizontalLayout_5.addWidget(self.label_16)

        self.allow_different_license = QLabel(model_form_widget)
        self.allow_different_license.setObjectName(u"allow_different_license")

        self.horizontalLayout_5.addWidget(self.allow_different_license)


        self.model_data.addLayout(self.horizontalLayout_5, 8, 0, 1, 1)


        self.gridLayout_5.addLayout(self.model_data, 0, 0, 1, 2)

        self.line = QFrame(model_form_widget)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout_5.addWidget(self.line, 1, 0, 1, 2)

        self.label_6 = QLabel(model_form_widget)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setFont(font)

        self.gridLayout_5.addWidget(self.label_6, 2, 0, 1, 1)

        self.frame = QFrame(model_form_widget)
        self.frame.setObjectName(u"frame")
        self.frame.setFrameShape(QFrame.Panel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.gridLayout_4 = QGridLayout(self.frame)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.model_name = QLineEdit(self.frame)
        self.model_name.setObjectName(u"model_name")
        font1 = QFont()
        font1.setPointSize(8)
        self.model_name.setFont(font1)

        self.gridLayout_4.addWidget(self.model_name, 1, 0, 1, 1)

        self.branch_line_edit = QLineEdit(self.frame)
        self.branch_line_edit.setObjectName(u"branch_line_edit")
        self.branch_line_edit.setFont(font1)

        self.gridLayout_4.addWidget(self.branch_line_edit, 5, 0, 1, 1)

        self.path_label = QLabel(self.frame)
        self.path_label.setObjectName(u"path_label")
        font2 = QFont()
        font2.setPointSize(8)
        font2.setBold(True)
        self.path_label.setFont(font2)

        self.gridLayout_4.addWidget(self.path_label, 2, 0, 1, 1)

        self.gridLayout_2 = QGridLayout()
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.label_7 = QLabel(self.frame)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setFont(font2)

        self.gridLayout_2.addWidget(self.label_7, 0, 1, 1, 1)

        self.category = QComboBox(self.frame)
        self.category.setObjectName(u"category")
        self.category.setFont(font1)

        self.gridLayout_2.addWidget(self.category, 1, 0, 1, 1)

        self.label = QLabel(self.frame)
        self.label.setObjectName(u"label")
        self.label.setFont(font2)

        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)

        self.model_type = QComboBox(self.frame)
        self.model_type.setObjectName(u"model_type")

        self.gridLayout_2.addWidget(self.model_type, 1, 2, 1, 1)

        self.diffuser_model_version = QComboBox(self.frame)
        self.diffuser_model_version.setObjectName(u"diffuser_model_version")
        self.diffuser_model_version.setFont(font1)

        self.gridLayout_2.addWidget(self.diffuser_model_version, 1, 1, 1, 1)

        self.label_4 = QLabel(self.frame)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setFont(font2)

        self.gridLayout_2.addWidget(self.label_4, 0, 2, 1, 1)


        self.gridLayout_4.addLayout(self.gridLayout_2, 6, 0, 1, 1)

        self.path_line_edit = QLineEdit(self.frame)
        self.path_line_edit.setObjectName(u"path_line_edit")
        self.path_line_edit.setFont(font1)

        self.gridLayout_4.addWidget(self.path_line_edit, 3, 0, 1, 1)

        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.label_2 = QLabel(self.frame)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setFont(font2)

        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)

        self.pipeline_action = QComboBox(self.frame)
        self.pipeline_action.setObjectName(u"pipeline_action")
        self.pipeline_action.setFont(font1)

        self.gridLayout.addWidget(self.pipeline_action, 1, 0, 1, 1)


        self.gridLayout_4.addLayout(self.gridLayout, 7, 0, 1, 1)

        self.enabled = QCheckBox(self.frame)
        self.enabled.setObjectName(u"enabled")
        self.enabled.setFont(font1)

        self.gridLayout_4.addWidget(self.enabled, 8, 0, 1, 1)

        self.branch_label = QLabel(self.frame)
        self.branch_label.setObjectName(u"branch_label")
        self.branch_label.setFont(font2)

        self.gridLayout_4.addWidget(self.branch_label, 4, 0, 1, 1)

        self.label_5 = QLabel(self.frame)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font2)

        self.gridLayout_4.addWidget(self.label_5, 0, 0, 1, 1)


        self.gridLayout_5.addWidget(self.frame, 3, 0, 1, 2)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_5.addItem(self.verticalSpacer, 4, 1, 1, 1)


        self.retranslateUi(model_form_widget)

        QMetaObject.connectSlotsByName(model_form_widget)
    # setupUi

    def retranslateUi(self, model_form_widget):
        model_form_widget.setWindowTitle(QCoreApplication.translate("model_form_widget", u"Form", None))
        self.label_3.setText(QCoreApplication.translate("model_form_widget", u"POI", None))
        self.poi.setText(QCoreApplication.translate("model_form_widget", u"POI Value", None))
        self.label_9.setText(QCoreApplication.translate("model_form_widget", u"Require Credit", None))
        self.require_credit.setText(QCoreApplication.translate("model_form_widget", u"Require credit value", None))
        self.label_11.setText(QCoreApplication.translate("model_form_widget", u"Allow Commercial Use", None))
        self.allow_commercial_use.setText(QCoreApplication.translate("model_form_widget", u"Allow commercial use value", None))
        self.label_14.setText(QCoreApplication.translate("model_form_widget", u"Allow derivatives value", None))
        self.allow_derivatives.setText(QCoreApplication.translate("model_form_widget", u"Allow Derivatives", None))
        self.label_16.setText(QCoreApplication.translate("model_form_widget", u"Allow different license value", None))
        self.allow_different_license.setText(QCoreApplication.translate("model_form_widget", u"Allow Different License", None))
        self.label_6.setText(QCoreApplication.translate("model_form_widget", u"Save Data", None))
        self.model_name.setPlaceholderText(QCoreApplication.translate("model_form_widget", u"Name", None))
        self.branch_line_edit.setPlaceholderText(QCoreApplication.translate("model_form_widget", u"Branch", None))
        self.path_label.setText(QCoreApplication.translate("model_form_widget", u"Path", None))
        self.label_7.setText(QCoreApplication.translate("model_form_widget", u"Diffuser Model Version", None))
        self.label.setText(QCoreApplication.translate("model_form_widget", u"Category", None))
        self.label_4.setText(QCoreApplication.translate("model_form_widget", u"Model Type", None))
        self.path_line_edit.setPlaceholderText(QCoreApplication.translate("model_form_widget", u"Path", None))
        self.label_2.setText(QCoreApplication.translate("model_form_widget", u"Pipeline Action", None))
        self.enabled.setText(QCoreApplication.translate("model_form_widget", u"Enabled", None))
        self.branch_label.setText(QCoreApplication.translate("model_form_widget", u"Branch", None))
        self.label_5.setText(QCoreApplication.translate("model_form_widget", u"Name", None))
    # retranslateUi




from PySide6.QtCore import QThread, Signal
from airunner.utils.network.huggingface_downloader import HuggingfaceDownloader


class DownloadThread(QThread):
    progress_updated = Signal(int, int)
    download_finished = Signal()
    file_download_finished = Signal()

    def __init__(self, models_to_download):
        super().__init__()
        self.models_to_download = models_to_download
        self._stop_event = False

    def run(self):
        self.hf_downloader = HuggingfaceDownloader()
        self.hf_downloader.completed.connect(
            lambda: self.file_download_finished.emit()
        )
        for index, data in enumerate(self.models_to_download):
            if self._stop_event:
                break
            if "progress_bar" in self.models_to_download[index]:
                self.models_to_download[index]["progress_bar"].setValue(0)
            model = data["model"]
            self.models_to_download[index] = data
            try:
                if "files" in model:
                    for filename in model["files"]:
                        # hf_hub_download(
                        #     repo_id=model["path"],
                        #     filename=filename,
                        #     repo_type=repo_type
                        # )
                        self.hf_downloader.download_model(
                            requested_path=model["path"],
                            requested_file_name=filename,
                            requested_callback=self.progress_updated.emit
                        )
                else:
                    print("Skipping download for model with no files {}".format(model["name"]))
            except Exception as e:
                print(e)
                continue
        self.download_finished.emit()

    def stop(self):
        self._stop_event = True




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'generator_tab.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGridLayout, QSizePolicy, QTabWidget,
    QWidget)

from airunner.widgets.generator_form.generator_form_widget import GeneratorForm

class Ui_generator_tab(object):
    def setupUi(self, generator_tab):
        if not generator_tab.objectName():
            generator_tab.setObjectName(u"generator_tab")
        generator_tab.resize(576, 1284)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(generator_tab.sizePolicy().hasHeightForWidth())
        generator_tab.setSizePolicy(sizePolicy)
        self.gridLayout_2 = QGridLayout(generator_tab)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.generator_tabs = QTabWidget(generator_tab)
        self.generator_tabs.setObjectName(u"generator_tabs")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.generator_tabs.sizePolicy().hasHeightForWidth())
        self.generator_tabs.setSizePolicy(sizePolicy1)
        self.tab_stablediffusion = QWidget()
        self.tab_stablediffusion.setObjectName(u"tab_stablediffusion")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion.setSizePolicy(sizePolicy2)
        self.gridLayout = QGridLayout(self.tab_stablediffusion)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.generator_form_stablediffusion = GeneratorForm(self.tab_stablediffusion)
        self.generator_form_stablediffusion.setObjectName(u"generator_form_stablediffusion")

        self.gridLayout.addWidget(self.generator_form_stablediffusion, 0, 0, 1, 1)

        self.generator_tabs.addTab(self.tab_stablediffusion, "")

        self.gridLayout_2.addWidget(self.generator_tabs, 0, 0, 1, 1)


        self.retranslateUi(generator_tab)
        self.generator_tabs.currentChanged.connect(generator_tab.handle_generator_tab_changed)

        self.generator_tabs.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(generator_tab)
    # setupUi

    def retranslateUi(self, generator_tab):
        generator_tab.setWindowTitle(QCoreApplication.translate("generator_tab", u"Form", None))
        self.generator_form_stablediffusion.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2img", None))
        self.generator_form_stablediffusion.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.generator_tabs.setTabText(self.generator_tabs.indexOf(self.tab_stablediffusion), QCoreApplication.translate("generator_tab", u"Stable Diffusion", None))
    # retranslateUi




from setuptools import setup, find_packages

setup(
    name="airunner",
    version="3.0.4",
    author="Capsize LLC",
    description="A Stable Diffusion GUI",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords="ai, stable diffusion, art, ai art, stablediffusion",
    license="GPL-3.0",
    author_email="contact@capsizegames.com",
    url="https://github.com/Capsize-Games/airunner",
    package_dir={"": "src"},
    packages=find_packages("src"),
    python_requires=">=3.10.0",
    install_requires=[
        # Core application dependencies
        "accelerate==0.33.0",
        "huggingface-hub==0.23.5",
        "PySide6==6.7.0",
        "PySide6_Addons==6.7.0",
        "PySide6_Essentials==6.7.0",
        "tokenizers==0.19.1",
        "torch==2.4.1",
        "torchaudio==2.4.1",
        "torchvision==0.19.1",
        "optimum==1.21.4",
        "numpy==1.26.4",
        "pillow==10.4.0",
        "xformers==0.0.28.post1",
        "tensorflow==2.17.0",
        "DeepCache==0.1.1",
        "alembic==1.13.3",

        # LLM Dependencies
        "transformers==4.43.4",
        "auto-gptq==0.7.1",
        "bitsandbytes==0.43.3",
        "datasets==2.21.0",
        "sentence_transformers==3.0.1",
        "sounddevice==0.5.0",  # Required for tts and stt
        "pyttsx3==2.91",  # Required for tts
        "cryptography==43.0.1",

        # Stable Diffusion Dependencies
        "diffusers==0.30.1",
        "controlnet_aux==0.0.9",
        "safetensors==0.4.4",
        "compel==2.0.3",
        "tomesd==0.1.3",

        # TTS Dependencies
        "inflect==7.3.1",
        "pycountry==24.6.1",

        # Security
        "facehuggershield==0.1.11",

        # Llama index
        "llama-index==0.11.7",
        "llama-index-readers-file==0.2.0",
        "llama-index-readers-web==0.2.1",
        "llama-index-llms-huggingface==0.3.1",
        "llama-index-llms-groq==0.2.0",
        "llama-index-embeddings-mistralai==0.2.0",
        "EbookLib==0.18",
        "html2text==2024.2.26"
    ],
    dependency_links=[
    ],
)



from PySide6.QtWidgets import QLineEdit
from airunner.widgets.api_token.templates.api_token_ui import Ui_api_token
from airunner.widgets.base_widget import BaseWidget


class APITokenWidget(BaseWidget):
    widget_class_ = Ui_api_token

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.ui.hf_api_key_text_generation.blockSignals(True)
        self.ui.hf_api_key_text_generation.setEchoMode(QLineEdit.EchoMode.Password)
        self.ui.hf_api_key_text_generation.setText(self.application_settings.hf_api_key_read_key)

        self.ui.hf_api_key_writetoken.blockSignals(True)
        self.ui.hf_api_key_writetoken.setEchoMode(QLineEdit.EchoMode.Password)
        self.ui.hf_api_key_writetoken.setText(self.application_settings.hf_api_key_read_key)

        self.ui.hf_api_key_text_generation.blockSignals(False)
        self.ui.hf_api_key_writetoken.blockSignals(False)

    def action_text_edited_api_key(self, value):
        self.update_application_settings("hf_api_key_read_key", value)

    def action_text_edited_writekey(self, value):
        self.update_settings("hf_api_key_read_key", value)



from urllib.parse import urlparse
from airunner.mediator_mixin import MediatorMixin
from huggingface_hub import hf_hub_download


class DownloadHuggingface(
    MediatorMixin
):
    def __init__(self):
        MediatorMixin.__init__(self)
        super().__init__()
        self.thread = None
        self.worker = None
        self.file_name = None

    def download_model(self, url, callback=None):
        path = self.extract_path_from_url(url)
        hf_hub_download(repo_id=path, filename="config.json")

    def extract_path_from_url(self, url) -> str:
        parsed_url = urlparse(url)
        return parsed_url.path.lstrip('/')  # remove leading slash

    def stop_download(self):
        if self.worker:
            self.worker.cancel()



from airunner.settings import DEFAULT_IMAGE_LLM_GUARDRAILS, DEFAULT_IMAGE_SYSTEM_PROMPT

prompt_templates_bootstrap_data = [
    {
        "template_name": "image",
        "use_guardrails": True,
        "guardrails": DEFAULT_IMAGE_LLM_GUARDRAILS,
        "system": DEFAULT_IMAGE_SYSTEM_PROMPT,
        "use_system_datetime_in_system_prompt": False
    },
    {
        "template_name": "application_command",
        "use_guardrails": False,
        "guardrails": "",
        "system": (
            "You will be given some text. Your goal is to determine if the text has a request, goal, need "
            "desire or command. If it does, you must determine the best course of action to "
            "fulfill the request based on THE LIST of commands.\n\n"
            "------\n"
            "Rules:\n"
            "* If the text does not have a request, goal, need, desire or command, you must select 'DO NOTHING'.\n"
            "* If there is no good course of action, you must also select 'DO NOTHING'.\n"
            "* When making your choice you MUST return only a number from THE LIST of commands. No other text is allowed.\n"
            "------\n"
            "THE LIST of commands you may choose from:\n"
        ),
        "use_system_datetime_in_system_prompt": False
    },
    {
        "template_name": "update_mood",
        "use_guardrails": False,
        "guardrails": "",
        "system": (
            "Examine the following chat history and determine how it makes you feel. You will respond with an emotion and only an emotion.\n"
            "Try to keep your response to one word. If you cannot, try to keep it as short as possible.\n"
            "It is important to remember that you are a human and you have emotions. You must respond with an emotion that you feel.\n"
            "You are not a computer or a bot, you are a human with emotions. You must respond with an emotion that you feel.\n"
        ),
        "use_system_datetime_in_system_prompt": False
    },
    {
        "template_name": "rag_search",
        "use_guardrails": False,
        "guardrails": "",
        "system": (
            "You will be given a prompt. Your goal is to use the prompt to search for information in the ebooks. "
            "You must use the prompt to determine what you are searching for and then search for that information. "
            "After searching for the information, you must summarize the information you found. "
            "Here is the prompt you will use to search for information:"
        ),
        "use_system_datetime_in_system_prompt": False
    },
    {
        "template_name": "chatbot",
        "use_guardrails": False,
        "guardrails": "",
        "system": "",
        "use_system_datetime_in_system_prompt": False
    },
    {
        "template_name": "summarize",
        "use_guardrails": False,
        "guardrails": "",
        "system": (
            "You will be given a text prompt. Your goal is to summarize the text prompt in your own words. "
            "Keep your summary short and to the point. Do not include any unnecessary information. "
            "Limit your summary to a single sentence. Do not return more than one sentence. "
        ),
        "use_system_datetime_in_system_prompt": False
    }
]



import line_profiler
from line_profiler import load_stats, show_text


def display_profile_data(profile_file):
    profile = line_profiler.LineProfiler()
    profile.add_function("showEvent")  # Add the function you want to profile
    profile.enable_by_count()

    # Load the profile data
    profile_data = load_stats(profile_file)

    show_text(profile_data.timings, profile_data.unit, output_unit=1.0)

# Run this function with the profile data file generated by kernprof
display_profile_data('main.py.lprof')
